/*
 * $Id: chanmodeseditor.ui.h,v 1.4 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 Julien Perrot <blackmore@no-log.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

void ChanModesEditor::setChannel( Channel *c )
{
    if (!c)
	return;
    chan = c;

    bool b;

    b = chan->isTopicOpOnly();
    opTopicCheckBox->setChecked(b);
    checkBoxes.insert("t", opTopicCheckBox);
    origValues["t"] = b;

    b = chan->isNoOutMessage();
    noExternalMessagesCheckBox->setChecked(b);
    checkBoxes.insert("n", noExternalMessagesCheckBox);
    origValues["n"] = b;

    b = chan->isInviteOnly();
    inviteOnlyCheckBox->setChecked(b);
    checkBoxes.insert("i", inviteOnlyCheckBox);
    origValues["i"] = b;

    b = chan->isModerated();
    moderatedCheckBox->setChecked(b);
    checkBoxes.insert("m", moderatedCheckBox);
    origValues["m"] = b;

    b = chan->isPrivate();
    privateCheckBox->setChecked(b);
    checkBoxes.insert("p", privateCheckBox);
    origValues["p"] = b;

    b = chan->isSecret();
    secretCheckBox->setChecked(b);
    checkBoxes.insert("s", secretCheckBox);
    origValues["s"] = b;

    origKey = chan->getKey();
    b = !origKey.isEmpty();
    keyCheckBox->setChecked(b);

    if (b) {
	    keyValue->setEnabled(true);
	    keyValue->setText(origKey);
    }

    b = chan->isLimited();
    origLimit = chan->getLimit();
    limitUsersCheckBox->setChecked(b);

    if (b) {
	    limitUsersValue->setEnabled(true);
	    limitUsersValue->setValue(origLimit);
    }

    bansListBox->insertStringList(chan->getBanMasks());
}

void ChanModesEditor::slotKeyCheckBoxChange()
{
    keyValue->setEnabled(keyCheckBox->isChecked());
}

void ChanModesEditor::slotLimitCheckBoxChange()
{
    limitUsersValue->setEnabled(limitUsersCheckBox->isChecked());
}

QStringList ChanModesEditor::getBanMasks()
{
    return bansListBox->items();
}

QString ChanModesEditor::getKey()
{
    return keyValue->text();
}

int ChanModesEditor::getLimit()
{
    return limitUsersValue->value();
}

QString ChanModesEditor::getChangedModes()
{
    return getModes(true);
}

QString ChanModesEditor::getModes(bool compareToOrig)
{
    QString plusModes = QString();
    QString minusModes = QString();
    QString modes = QString();
    QDictIterator<QCheckBox> it (checkBoxes);
    QString l;
    QCheckBox * cBox;
    bool b;

    for ( ; it.current(); ++it) {
	cBox = it.current();
	b = cBox->isChecked();
	l = it.currentKey();

	if (compareToOrig) {
	    if (b == origValues[l]) {
		continue;
	    }
	}

	if (b)  { // +
	    plusModes += l;
	 } else {
	     minusModes += l;
	 }
    }

    if (!plusModes.isEmpty()) {
	modes += "+" + plusModes;
    }

    if (!minusModes.isEmpty()) {
	modes += "-" + minusModes;
    }

    return modes;
}

bool ChanModesEditor::hasKey()
{
	return keyCheckBox->isChecked();
}

bool ChanModesEditor::hasLimit()
{
	return limitUsersCheckBox->isChecked();
}
