/*
 * $Id: channel.cpp,v 1.47 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include <config.h>

#include <ktoolbarbutton.h>

#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qptrlist.h>
#include <qstringlist.h>
#include <stdio.h>
#include <stdlib.h>

#include <kfiledialog.h>

#include <kfontdialog.h>
#include <kstandarddirs.h>
#include <qpainter.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <krootpixmap.h>
#include <kstdaction.h>
#include <klineeditdlg.h>
#include <kdebug.h>
#include <kipc.h>
#include <dcopclient.h>

#include <klocale.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <signal.h>
#include <dirent.h>
#include <fcntl.h>
#include <assert.h>

#include <kiconloader.h>
#include <kstringhandler.h>
#include <ktip.h>

#include "kwirc.h"
#include "kwircpreferences.h"
#include "server.h"
#include "channel.h"
#include "command.h"
#include <netwm.h>
#include "config.h"
#ifdef FEAT_RUBY
#include "rubyscriptmanager.h"
#endif

#include "kwircsettings.h"

unsigned int Channel::timeout_seed = 1;

Channel::Channel(Server *s, const QString &n) :
	Destination(s, n, TYPE_CHANNEL),
	modesFlag(0),
	whoDisplay(true),
	key(""),
	limit(-1)
{
	whoTimer = new QTimer(this);
	connect(whoTimer, SIGNAL(timeout()), this, SLOT(whoTimerTrigger()) );
	//whoTimer->start(60000 * timeout_seed++, false);
#ifdef FEAT_RUBY
	setRubyInstance(RubyScriptManager::getRubyChannelInstance(this));
#endif
}

Channel::~Channel()
{
	Nick *nick;
    	for (nick = nicks.first(); nick; nick = nicks.next())
		nick->noMoreIn(this);
}

void Channel::cleanup()
{
	modesFlag = 0;
	nicks.clear();
	voicedNicks.clear();
	opNicks.clear();
}

void Channel::sync()
{
	updateNames();
}

void Channel::setInChannel(const QString &nicklist)
{
	Nick *n;
	QStringList qsl = QStringList::split(' ',nicklist);
	QStringList::iterator it;
	for ( it = qsl.begin(); it != qsl.end(); ++it ) {
		if((*it).startsWith("@")) {
			opNicks.append(n = getServer()->nickFromString(*it));
		} else if ((*it).startsWith("+")) {
			voicedNicks.append(n
					= getServer()->nickFromString(*it));
		}
		nicks.append(n = getServer()->nickFromString(*it));
		n->isIn(this);
	}
}

void Channel::setWhoSet(const QString &w, const QString &whe)
{
	whoset = w;
	when = whe;
}

void Channel::whois(const QString &nick)
{
	Nick *n = getServer()->nickFromString(nick);
	if(!n)
		return;
	getServer()->sendWhois(n->getName());
}

void Channel::setWhoDisplay(bool enable)
{
	whoDisplay = enable;
}

/* this is end of who, we now have the away status of all nicks in this
 * channel, thus we must update the nicklist */
void Channel::endOfWho()
{
	updateNames();
}

void Channel::recvWho(Nick *n, const QString &whoRepl)
{
	if (whoDisplay)
		emit log(LOG_WHO, getStringId(), n->getStringId(), whoRepl);
}

void Channel::recvModeSetLimit(const QString &s)
{
	modesFlag |= MODE_LIMIT;
	limit = s.toInt();
}

void Channel::recvModeUnsetLimit()
{
	modesFlag &= ~MODE_LIMIT;
	limit = -1;
}

void Channel::recvModeAddBanMask(const QString &s)
{
	if(banMasks.contains(s))
		return;
	banMasks.append(s);
}

void Channel::recvModeRemoveBanMask(const QString &s)
{
	if(!banMasks.contains(s))
		return;
	banMasks.remove(s);
}

void Channel::recvModeOp(const QString &nick)
{
	Nick *n = getServer()->nickFromString(nick);
	if(!nicks.contains(n))
		return;
	if(opNicks.contains(n))
		return;
	opNicks.append(n);
	updateNames();
}

void Channel::recvModeDeop(const QString &nick)
{
	Nick *n = getServer()->nickFromString(nick);
	if(!nicks.contains(n))
		return;
	if(!opNicks.contains(n))
		return;
	opNicks.remove(n);
	updateNames();
}

void Channel::recvModeVoice(const QString &nick)
{
	Nick *n = getServer()->nickFromString(nick);
	if(!n)
		return;
	if(!nicks.contains(n))
		return;
	if(voicedNicks.contains(n))
		return;
	voicedNicks.append(n);
	updateNames();
}

void Channel::recvModeDevoice(const QString &nick)
{
	Nick *n = getServer()->nickFromString(nick);
	if(!n)
		return;
	if(!nicks.contains(n))
		return;
	voicedNicks.remove(n);
	updateNames();
}

void Channel::recvModeSetKey(const QString &newkey)
{
	key = newkey;
}

void Channel::recvModeUnsetKey()
{
	key = QString();
}


void Channel::updateNames()
{
	emit channelUsersChange(opNicks, voicedNicks, nicks);
}

void Channel::addNick(Nick *n)
{
	nicks.append(n);
	updateNames();
}

void Channel::removeNick(Nick *n)
{
	opNicks.remove(n);
	voicedNicks.remove(n);
	nicks.remove(n);
	updateNames();
}

void Channel::youJoined()
{
	emit log(LOG_YOU_JOIN, getStringId(), QString::null, QString::null);

	/* XXX: Timer does it now
	setWhoDisplay(false);
	getServer()->sendWho(getStringId());
	*/
#ifdef FEAT_RUBY
	RubyScriptManager::notifyYouJoined(this);
#endif
	// Request RPL_CHANNELMODEIS
	getServer()->putserv("MODE " + getStringId() + " \r\n");
	// Request ban list (RPL_BANLIST)
	getServer()->putserv("MODE " + getStringId() + " +b\r\n");
}

void Channel::youLeft(const QString &message)
{
	emit log(LOG_PART, getStringId(), i18n("you"), message);
	emit left();
#ifdef FEAT_RUBY
	RubyScriptManager::notifyYouLeft(this, message);
#endif
}

void Channel::youWereKicked(Nick *kicker, const QString &message)
{
	emit log(LOG_KICK, getStringId(), i18n("you"),
			kicker->getStringId() + ":" + message);
	emit left();
#ifdef FEAT_RUBY
	RubyScriptManager::notifyYouWereKicked(this, kicker, message);
#endif
}

void Channel::recvPrivmsg(Nick *nick, const QString &message)
{
	emit log(LOG_PRIVMSG, getStringId(), nick->getName(), message);

#ifdef FEAT_RUBY
	RubyScriptManager::notifyPublic(this, nick, message);
#endif
}

void Channel::recvNotice(Nick *nick, const QString &message)
{
	emit log(LOG_NOTICE, getStringId(), nick->getName(), message);
#ifdef FEAT_RUBY
	RubyScriptManager::notifyPublicNotice(this, nick, message);
#endif
}

void Channel::recvCtcp(Nick *nick, const QString &ctcp,
			const QString &message)
{
	emit log(LOG_CTCP, getStringId(), nick->getName(),
			QString("%1 %2").arg(ctcp).arg(message));
#ifdef FEAT_RUBY
	RubyScriptManager::notifyPublicCtcp(this, nick, ctcp, message);
#endif
}

void Channel::recvCtcpNotice(Nick *nick, const QString &ctcp,
		const QString &message)
{
	emit log(LOG_NOTICE, getStringId(), nick->getName(),
			QString("%1 %2").arg(ctcp).arg(message));
#ifdef FEAT_RUBY
	RubyScriptManager::notifyPublicCtcpNotice(this, nick, ctcp, message);
#endif
}

void Channel::recvAction(Nick *nick, const QString &message)
{
	emit log(LOG_ACTION, getStringId(), nick->getName(), message);
#ifdef FEAT_RUBY
	RubyScriptManager::notifyPublicAction(this, nick, message);
#endif
}

void Channel::recvMode(Nick *nick, const QString &mode)
{
	if (nick)
		emit log(LOG_MODE, getStringId(), nick->getName(), mode);

	QString modeChars = mode.section(' ', 0, 0);
	QStringList argList = QStringList::split(' ', mode.section(' ', 1));
	QStringList::Iterator currentArg = argList.begin();
	bool add = true;

	for(unsigned int i = 0; i < modeChars.length(); i++) {
		switch(modeChars.at(i).latin1()) {
			case '+':
				add = true;
				break;
			case '-':
				add = false;
				break;
			case 'o':
				if(add)
					recvModeOp(*currentArg);
				else
					recvModeDeop(*currentArg);
				currentArg++;
				break;
			case 'p':
				if(add)
					modesFlag |= MODE_PRIVATE;
				else
					modesFlag &= ~MODE_PRIVATE;
				break;
			case 's':
				if(add)
					modesFlag |= MODE_SECRET;
				else
					modesFlag &= ~MODE_SECRET;
				break;
			case 'i':
				if(add)
					modesFlag |= MODE_INVITE_ONLY;
				else
					modesFlag &= ~MODE_INVITE_ONLY;
				break;
			case 't':
				if(add)
					modesFlag |= MODE_TOPIC;
				else
					modesFlag &= ~MODE_TOPIC;
				break;
			case 'n':
				if(add)
					modesFlag |= MODE_NO_OUT;
				else
					modesFlag &= ~MODE_NO_OUT;
				break;
			case 'm':
				if(add)
					modesFlag |= MODE_MODERATED;
				else
					modesFlag &= ~MODE_MODERATED;
				break;
			case 'l':
				if(add) {
					recvModeSetLimit(*currentArg);
					currentArg++;
				} else
					recvModeUnsetLimit();
				break;
			case 'b':
				// a listing of banmasks is requested
				if(add)
					recvModeAddBanMask(*currentArg);
				else
					recvModeRemoveBanMask(*currentArg);
				currentArg++;
				break;
			case 'v':
				if(add)
					recvModeVoice(*currentArg);
				else
					recvModeDevoice(*currentArg);
				currentArg++;
				break;
			case 'k':
				if(add) {
					recvModeSetKey(*currentArg);
					currentArg++;
				} else
					recvModeUnsetKey();
				break;
			default:
				break;
		}
	}

#ifdef FEAT_RUBY
	if (nick)
		RubyScriptManager::notifyModeChanged(this, nick, mode);
#endif
}


void Channel::recvJoin(Nick *n)
{
	addNick(n);
	emit log(LOG_JOIN, getStringId(), n->getName(),	n->getAuxInfo());
#ifdef FEAT_RUBY
	RubyScriptManager::notifyJoin(this, n);
#endif
}

void Channel::recvInvite(Nick *inviter, Nick *invited)
{
	emit log(LOG_INVITE, getStringId(), inviter->getName(),
			invited->getName());
#ifdef FEAT_RUBY
	RubyScriptManager::notifyInvite(this, inviter, invited);
#endif
}

void Channel::recvKill(Nick *killed, const QString &message)
{
	removeNick(killed);
	emit log(LOG_KILL, getStringId(), killed->getName(), message);
#ifdef FEAT_RUBY
	RubyScriptManager::notifyKill(this, killed, message);
#endif
}

void Channel::recvKick(Nick *kicked, Nick *kicker, const QString &message)
{
	removeNick(kicked);
	emit log(LOG_KICK, getStringId(), kicker->getName(),
			QString("%1 %2").arg(kicked->getName()).arg(message));
#ifdef FEAT_RUBY
	RubyScriptManager::notifyKicked(this, kicked, kicker, message);
#endif
}

void Channel::recvPart(Nick *n, const QString &message)
{
	removeNick(n);
	emit log(LOG_PART, getStringId(), n->getStringId(), message);
#ifdef FEAT_RUBY
	RubyScriptManager::notifyPart(this, n, message);
#endif
}

void Channel::recvQuit(Nick *n, const QString &message)
{
	removeNick(n);
	emit log(LOG_QUIT, getStringId(), n->getStringId(), message);
}

void Channel::recvNick(Nick *n, const QString &oldnick)
{
	emit log(LOG_NICK, getStringId(), oldnick, n->getName());
	updateNames();
}

void Channel::recvTopic(const QString &who, const QString &top)
{
	whoset = who;
	emit log(LOG_TOPIC, getStringId(), who, top);
	emit newTopic(top);
#ifdef FEAT_RUBY
	RubyScriptManager::notifyTopic(this, who, top);
#endif
}

void Channel::ctcp(const QString &nick, const QString &stuff)
{
	getServer()->sendCtcp(nick, stuff);
}

void Channel::topic(const QString &topic)
{
	getServer()->sendTopic(getStringId(), topic);
}

void Channel::nickPrivMode(const QString &nick, const QChar &c, bool add)
{
	QStringList sl = QStringList::split(' ', nick);
	int n = sl.count();
	QString ops = add ? "+" : "-";
	for (int i = 0; i < n; i++)
		ops += c;

	getServer()->sendMode(getStringId(), ops, nick);
}

void Channel::op(const QString &nick)
{
	nickPrivMode(nick, 'o', true);
}
void Channel::deop(const QString &nick)
{
	nickPrivMode(nick, 'o', false);
}
void Channel::voice(const QString &nick)
{
	nickPrivMode(nick, 'v', true);
}
void Channel::devoice(const QString &nick)
{
	nickPrivMode(nick, 'v', false);
}
void Channel::setKey(const QString &akey)
{
	if (key != akey)
		modes("+k", akey);
}
void Channel::unsetKey()
{
	if (!key.isEmpty())
		modes("-k", key);
}
void Channel::ban(const QString &mask)
{
	modes("+b ", mask);
}
void Channel::unban(const QString &mask)
{
	if (banMasks.contains(mask))
		modes("-b", mask);
}
void Channel::part()
{
	getServer()->sendPart(getStringId(),
			KwircSettings::partMessage());
}
void Channel::part(const QString &reason)
{
	getServer()->sendPart(getStringId(), reason);
}
void Channel::setLimit(int i)
{
	if (i != limit)
		modes("+l", QString("%1").arg(i));
}

void Channel::unsetLimit()
{
	if (isLimited())
		modes("-l", "");
}

const QString & Channel::getKey()
{
	return key;
}

const QStringList & Channel::getBanMasks()
{
	return banMasks;
}

int Channel::getLimit()
{
	return limit;
}

bool Channel::isPrivate()
{
	return ((modesFlag & MODE_PRIVATE) != 0);
}

bool Channel::isSecret()
{
	return ((modesFlag & MODE_SECRET) != 0);
}

bool Channel::isInviteOnly()
{
	return ((modesFlag & MODE_INVITE_ONLY) != 0);
}

bool Channel::isTopicOpOnly()
{
	return ((modesFlag & MODE_TOPIC) != 0);
}

bool Channel::isNoOutMessage()
{
	return ((modesFlag & MODE_NO_OUT) != 0);
}

bool Channel::isModerated()
{
	return ((modesFlag & MODE_MODERATED) != 0);
}

bool Channel::isLimited()
{
	return ((modesFlag & MODE_LIMIT) != 0);
}

void Channel::modes(const QString &m, const QString &args)
{
	getServer()->sendMode(getStringId(), m, args);
}

void Channel::setBanMasks(QStringList masks)
{
	// Ban new masks
	for (QStringList::Iterator it = masks.begin(); it != masks.end();
			++it ) {
		if(!banMasks.contains(*it)) {
			ban(*it);
		}
	}

	// Unban removed masks
	for (QStringList::Iterator it = banMasks.begin(); it != banMasks.end();
			++it ) {
		if(!masks.contains(*it)) {
			unban(*it);
		}
	}
}

void Channel::setInitialModes(const QString & modes)
{
	recvMode(NULL, modes);
}

void Channel::addInitialBanMask( const QString & mask)
{
	recvModeAddBanMask(mask);
}

void Channel::whoTimerTrigger()
{
	setWhoDisplay(false);
	getServer()->sendWho(getStringId());
}

const QPtrList<Nick> &Channel::getNicks()
{
	return nicks;
}
