/*
 * $Id: channel.h,v 1.24 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#ifndef CHANNEL_H
#define CHANNEL_H

#include <qptrlist.h>
#include <qstringlist.h>

#include "kwirclog.h"
#include "destination.h"

class ServerElem;
class Destination;
class Server;
class Nick;

class QStringList;

class Channel : public Destination
{
	Q_OBJECT
	enum {
		MODE_PRIVATE = 0x1,
		MODE_SECRET = 0x1<<1,
		MODE_INVITE_ONLY = 0x1<<2,
		MODE_TOPIC = 0x1<<3,
		MODE_NO_OUT = 0x1<<4,
		MODE_MODERATED = 0x1<<5,
		MODE_LIMIT = 0x1<<6
	};
private:
	unsigned int modesFlag;
	QString name;


	QString whoset,when;
	QPtrList<Nick> voicedNicks;	// voiced
	QPtrList<Nick> opNicks; 	// ops
	QPtrList<Nick> nicks;		// All nicks

	QStringList banMasks;
	bool whoDisplay;
	QString key;
	int limit;

	QTimer *whoTimer;
	static unsigned int timeout_seed;

	void recvModeAddBanMask(const QString &s);
	void recvModeRemoveBanMask(const QString &s);
	void recvModeOp(const QString &nick);
	void recvModeDeop(const QString &nick);
	void recvModeVoice(const QString &nick);
	void recvModeDevoice(const QString &nick);
	void recvModeSetKey(const QString &newkey);
	void recvModeUnsetKey();
	void recvModeSetLimit(const QString &s);
	void recvModeUnsetLimit();
	void nickPrivMode(const QString &nick, const QChar &c, bool add);
public: // Public attributes
	Channel(Server *, const QString &);
	virtual ~Channel();

	void cleanup();
	void addNick(Nick *n);
	void removeNick(Nick *n);
	void setWhoSet(const QString &w, const QString &whe);
	void setInChannel(const QString &nicklist);
	void sync();
	void updateNames();
	void youJoined();
	void setWhoDisplay(bool enable);
	void endOfWho();

	void youLeft(const QString &);
	void youWereKicked(Nick *kicker, const QString &);

	void recvWho(Nick *nick, const QString &whoRepl);
	void recvCtcp(Nick *nick, const QString &ctcp,
		      const QString &message);
	void recvPrivmsg(Nick *nick, const QString &message);
	void recvCtcpNotice(Nick *, const QString &ctcp,
			const QString &messge);
	void recvAction(Nick *nick, const QString &message);
	void recvNotice(Nick *nick, const QString &message);
	void recvMode(Nick *nick, const QString &mode);
	void recvJoin(Nick *n);
	void recvInvite(Nick *inviter, Nick *invited);
	void recvKill(Nick *n, const QString &message);
	void recvKick(Nick *k, Nick*kicker,const QString &message);
	void recvQuit(Nick *n, const QString &message);
	void recvPart(Nick *n, const QString &message);
	void recvNick(Nick *n, const QString &oldnick);
	void recvTopic(const QString &, const QString &);

	const QString &getKey();
	const QStringList &getBanMasks();
	int getLimit();
	bool isPrivate();
	bool isSecret();
	bool isInviteOnly();
	bool isTopicOpOnly();
	bool isNoOutMessage();
	bool isModerated();
	bool isLimited();

	void setInitialModes(const QString & );
	void addInitialBanMask( const QString & );
	const QPtrList<Nick> &getNicks();
signals:
	void channelUsersChange(const QPtrList<Nick> &ops,
				const QPtrList<Nick> &voiced,
				const QPtrList<Nick> &all);
	void newTopic(const QString &);
	void left();
public slots:
	void setLimit(int);
	void unsetLimit();
	void ban(const QString &s);
	void unban(const QString &s);
	void whois(const QString &nick);
	void op(const QString &nick);
	void deop(const QString &nick);
	void voice(const QString &nick);
	void devoice(const QString &nick);
	void setKey(const QString &newkey);
	void unsetKey();
	void ctcp(const QString &nick, const QString &arg);
	void topic(const QString &);
	void part(const QString &);
	void part();
	void modes(const QString &, const QString &);
	void setBanMasks(QStringList);
	void whoTimerTrigger();
};

#endif
