/*
 * $Id: command.cpp,v 1.52 2004/03/10 16:40:59 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "config.h"
#include "command.h"
#include "channel.h"
#include "server.h"
#include "kwirc.h"
#include <qdir.h>
#include <qmap.h>
#include <kstandarddirs.h>
#include <klocale.h>
#ifdef FEAT_RUBY
#include "rubyscriptmanager.h"
#endif
#include "server.h"
#include "chanmodeseditor.h"
#include "ignorelisteditor.h"
#include "kwircdebug.h"
#include <kdebug.h>

CommandNickItem::~CommandNickItem()
{
}

CommandItem::~CommandItem()
{
}

#define CommandItemChild(cmd) class Command##cmd : public CommandItem  { \
	public:\
		virtual void execute(ServerElem *s, const QString &);\
}
#define CommandNickItemChild(cmd) class Command##cmd \
: public CommandNickItem  { \
	public:\
		virtual void execute(ServerElem *s, const QString &);\
}

CommandItemChild(Away);
CommandNickItemChild(Me);
CommandItemChild(Topic);
CommandItemChild(Join);
CommandItemChild(Part);
CommandItemChild(Nick);
CommandItemChild(Who);
CommandNickItemChild(Whois);
CommandNickItemChild(Query);
CommandNickItemChild(Op);
CommandNickItemChild(Deop);
CommandNickItemChild(Voice);
CommandNickItemChild(Devoice);
CommandItemChild(Quote);
CommandItemChild(Mode);
CommandItemChild(Quit);
CommandItemChild(Say);
CommandItemChild(Clear);
CommandItemChild(ClearAll);
CommandItemChild(ClearAllChans);

#ifdef FEAT_RUBY
class CommandScript : public CommandItem
{
	private:
		QStringList subCommands;
	public:
		CommandScript();
		virtual QStringList complete(ServerElem *, const QString &);
		virtual void execute(ServerElem *se, const QString &);
};
#endif
class CommandTheme : public CommandItem
{
	public:
		virtual void execute(ServerElem *se, const QString &);
		virtual QStringList complete(ServerElem *, const QString &);
};
class CommandCtcp : public CommandItem
{
	public:
		virtual void execute(ServerElem *se, const QString &);
		virtual bool nickComplete(ServerElem *se, const QString &);
};
class CommandMsg : public CommandItem
{
	public:
		virtual void execute(ServerElem *se, const QString &);
		virtual bool nickComplete(ServerElem *se, const QString &);
};
class CommandIgnore : public CommandItem
{
public:
	virtual void execute(ServerElem *se, const QString &);
	virtual bool nickComplete(ServerElem *se, const QString &);
	virtual QStringList complete(ServerElem *,const QString &);
};
class CommandUnIgnore : public CommandItem
{
public:
	virtual void execute(ServerElem *se, const QString &);
	virtual QStringList complete(ServerElem *,const QString &);
};
class CommandServer : public CommandItem
{
public:
	virtual void execute(ServerElem *se, const QString &);
	virtual QStringList complete(ServerElem *,const QString &);
};
class CommandNewServer : public CommandItem
{
public:
	virtual void execute(ServerElem *se, const QString &);
	virtual QStringList complete(ServerElem *,const QString &);
};
class CommandHelp : public CommandItem
{
public:
	virtual void execute(ServerElem *se, const QString &);
	virtual QStringList complete(ServerElem *,const QString &);
};

bool CommandItem::nickComplete(ServerElem *, const QString &)
{
	return false;
}

bool CommandNickItem::nickComplete(ServerElem *, const QString &)
{
	return true;
}

#ifdef FEAT_RUBY
void CommandScript::execute(ServerElem *, const QString &txt)
{
	if (txt.section(' ', 1, 1).lower().compare("load") == 0) {
		RubyScriptManager::loadScript(txt.section(' ', 2, 2));
	}
}
#endif

void CommandAway::execute(ServerElem *se, const QString &t)
{
	if (!se)
		return;
	QString str = t.section(' ', 1);
	se->getServer()->sendAway(str);
}

void CommandMe::execute(ServerElem *se, const QString &txt)
{
	if (!se || !se->isDestination())
		return;
	QString arg = txt.section(' ', 1);
	if (se->isDestination())
		((Destination *)se)->sendAction(arg);
}
void CommandQuit::execute(ServerElem *se, const QString &txt)
{
	if (!se)
		return;
	QString str = txt.section(' ', 1);
	se->getServer()->sendQuit(str);
}
void CommandSay::execute(ServerElem *se, const QString &txt)
{
	if (!se || !se->isDestination())
		return;
	QString str = txt.section(' ', 1);
	((Destination *)se)->sendPrivmsg(str);
}
void CommandClear::execute(ServerElem *se, const QString &)
{
	if (!se || !se->isDestination())
		return;
	se->clear();
}
void CommandClearAll::execute(ServerElem *se, const QString &)
{
	se->getServer()->clearAll();
}
void CommandClearAllChans::execute(ServerElem *se, const QString &)
{
	se->getServer()->clearAllChans();
}
void CommandTopic::execute(ServerElem *se, const QString &txt)
{
	if (!se || se->getType() != ServerElem::TYPE_CHANNEL)
		return;
	QString arg = txt.section(' ', 1);
	((Channel *)se)->topic(arg);
}
void CommandJoin::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	se->getServer()->sendJoin(command.section(' ', 1, 1),
			command.section(' ', 2, 2));
}

void CommandPart::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	if (se->getType() != ServerElem::TYPE_CHANNEL) {
		((Channel*)se)->part(command.section(' ', 1));
	} else {
		QString chan = command.section(' ', 1, 1);
		if (chan.isEmpty())
			return;
		se->getServer()->sendPart(chan, command.section(' ', 2));
	}
}

void CommandTheme::execute(ServerElem *, const QString &command)
{
	Command::emitLoadTheme(command.section(' ', 1));
}

void CommandNick::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	se->getServer()->sendNick(command.section(" ",-1));
}

void CommandWhois::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	QString arg = command.section(' ', -1);
	if (arg.isEmpty()) {
		if (se->getType() == ServerElem::TYPE_NICK)
			se->getServer()->sendWhois(se->getStringId());
		else
			return;
	}
	se->getServer()->sendWhois(arg);
}

void CommandWho::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	se->getServer()->sendWho(command.section(" ",-1));
}

void CommandQuery::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	Nick *n = se->getServer()->nickFromString(command.section(' ',-1));
	Command::emitOpenQuery(n);
}

void CommandOp::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	if (se->getType() == ServerElem::TYPE_CHANNEL)
		((Channel *)se)->op(command.section(' ', 1));
}
void CommandDeop::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	if (se->getType() == ServerElem::TYPE_CHANNEL)
		((Channel*)se)->deop(command.section(' ', 1));
}
void CommandVoice::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	if (se->getType() == ServerElem::TYPE_CHANNEL)
		((Channel *)se)->voice(command.section(' ', 1));
}
void CommandDevoice::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	if (se->getType() == ServerElem::TYPE_CHANNEL)
		((Channel *)se)->devoice(command.section(' ', 1));
}

void CommandCtcp::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	se->getServer()->sendCtcp(command.section(' ', 1, 1),
				command.section(' ', 2, 2));
}

void CommandMsg::execute(ServerElem *se, const QString &command)
{
	Nick *n = se->getServer()->nickFromString(command.section(' ',1,1));
	Command::emitOpenQuery(n);
	n->input(command.section(' ',2));
}

void CommandServer::execute(ServerElem *, const QString &command)
{
	QString host,username,pass, nick;
	int port;
	bool ok;
	bool ssl = false;
	QStringList arg = QStringList::split(' ', command);
	if ((*arg.at(1)) == "-ssl") {
		ssl = true;
		arg.remove(arg.first());
	}


	host = *arg.at(1);
	port = (*arg.at(2)).toInt(&ok);
	pass = *arg.at(3);

	if(!ok)
		port = 6667;

	Command::emitChangeServer(host, port, pass, ssl);
}

void CommandNewServer::execute(ServerElem *, const QString &command)
{
	QString host,username,pass, nick;
	int port;
	bool ok;
	bool ssl = false;
	QStringList arg = QStringList::split(' ', command);
	if ((*arg.at(1)) == "-ssl") {
		ssl = true;
		arg.remove(arg.first());
	}


	host = *arg.at(1);
	port = (*arg.at(2)).toInt(&ok);
	pass = *arg.at(3);

	if(!ok)
		port = 6667;

	Command::emitNewServer(host, port, pass, ssl);
}

void CommandQuote::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	QString cmd;
	cmd = command.section(' ', 1);
	cmd.append("\r\n");
	se->getServer()->putserv(cmd);
}

// /ignore nohar!*@* -chan -priv -notice
void CommandIgnore::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;
	Server * server = se->getServer();
	bool chan, priv, notice, invite;

	QString arg = command.section(' ', 1);

	QString mask = arg.section(' ', 0, 0);
	QString paramList = arg.section(' ', 1);

	if (mask.isEmpty()) {
		Command::log(i18n("Ignore list"),
			     i18n("No mask or nick provided"));
		return;
	}

	if (paramList.contains("-all") || paramList.isEmpty()) {
		chan = priv = notice = invite = true;
	} else {
		chan = paramList.contains("-chan");
		priv = paramList.contains("-priv");
		notice = paramList.contains("-notice");
		invite = paramList.contains("-invite");
	}

	server->addIgnoreMask(mask, chan, priv, notice, invite);
	server->saveIgnoreList();
}

void CommandHelp::execute(ServerElem *,  const QString &command )
{
	QString arg = command.section( ' ', 1 );
	if ( arg.length() == 0 ) {
		arg = QString( "help" );
	}
	Command::showHelp( arg.lower().prepend( "/" ) );
}

void CommandUnIgnore::execute(ServerElem *se, const QString &command)
{
	if (!se)
		return;

	QString arg = command.section(' ', 1);
	if (!arg.isEmpty())
		se->getServer()->removeIgnoreMask(arg);
}

void CommandMode::execute(ServerElem *se, const QString &)
{
	if (!se)
		return;
	//TODO
}

QStringList CommandItem::complete(ServerElem *, const QString &)
{
	return QStringList();
}

#ifdef FEAT_RUBY
CommandScript::CommandScript()
{
	subCommands = QStringList::split(',', "load"); // XXX:code unload
}

QStringList CommandScript::complete(ServerElem *, const QString &str)
{
	if (str.contains(QChar(' ')) == 1) {
		return subCommands;
	}
	QStringList scripts;
	if (str.section(' ', 1, 1).lower().compare("load") == 0) {
		QString path = str.section(' ', 2);
		if (path.find('/') == -1) {
			path = QString::null;
		}
		QStringList qsl = KGlobal::dirs()->findDirs("data",
				"kwirc/scripts");
		for ( QStringList::Iterator it = qsl.begin(); it != qsl.end();
				++it ) {
			QDir files(*it);
			files.setFilter(QDir::Files);
			files.setNameFilter("*.rb");
			files.setSorting(QDir::Name);
			scripts += files.entryList();
		}
	}
	return scripts;
}
#endif

QStringList CommandTheme::complete(ServerElem *, const QString &arg)
{
	QString path = arg.section(' ', 1);
	if (path.find('/') == -1) {
		path = QString::null;
	}
	QStringList qsl = KGlobal::dirs()->findDirs("data", "kwirc/themes");
	QStringList themes;
	for ( QStringList::Iterator it = qsl.begin(); it != qsl.end(); ++it ) {
		QDir files(*it);
		files.setFilter(QDir::Files);
		files.setNameFilter("*.theme");
		files.setSorting(QDir::Name);
		themes += files.entryList();
	}
	return themes;
}

QStringList CommandHelp::complete(ServerElem *se, const QString &)
{
	QStringList commands = Command::getCompletion(se, QString::null );
	QStringList res;
	for ( QStringList::Iterator it = commands.begin(); it != commands.end(); ++it ) {
		QString command = *it;
		if ( command.length() != 0 ) {
			if ( command[0] == '/' ) {
				command.remove( 0, 1 );
			}
			res.append( command );
		}
	}
	return res;
}

bool CommandCtcp::nickComplete(ServerElem *, const QString &txt)
{
	if (QStringList::split(' ', txt).size() <= 2)
		return true;
	return false;
}

bool CommandMsg::nickComplete(ServerElem *, const QString &txt)
{
	if (QStringList::split(' ', txt).size() <= 2)
		return true;
	return false;
}


bool CommandIgnore::nickComplete(ServerElem *, const QString &)
{
	return false;
}

QStringList CommandIgnore::complete(ServerElem *, const QString &txt)
{
	int size = QStringList::split(' ', txt).size();
	if (size <= 2) {
		return Command::getNicks();
	} else {
		return QStringList::split(",",
				"-all,-chan,-priv,-notice,""-ctcp,"
				"-invite");
	}
}

QStringList CommandUnIgnore::complete(ServerElem *se, const QString &)
{
	if (!se)
		return QStringList();
	QStringList res;
	Server * server = se->getServer();
	IgnoreList l = server->getIgnoreList();
	IgnoreList::Iterator it;
	for (it = l.begin(); it != l.end(); ++it)
		res.append(it.key());

	return res;
}

QStringList CommandNewServer::complete(ServerElem *, const QString &command)
{
	if (QStringList::split(' ', command).size() < 2)
		return QStringList::split(',', "-ssl");
	return QStringList();
}

QStringList CommandServer::complete(ServerElem *, const QString &command)
{
	if (QStringList::split(' ', command).size() < 2)
		return QStringList::split(',', "-ssl");
	return QStringList();
}

QStringList Command::commandComp;
QDict<CommandItem> Command::commands;
QStringList Command::nullStringList;
QStringList Command::nicks;
QMap<QString, QString> Command::helpStrings;

Command::Command()
{
	nullStringList = QStringList();

	registerCommand("/away", new CommandAway(),
			i18n( "/away <reason> : Marks you away"));
	registerCommand("/me", new CommandMe(),
			i18n( "/me message : Sends a CTCP ACTION to the current channel or query"));
	registerCommand("/join", new CommandJoin(),
			i18n( "/join channels : Joins the specified channels."));
	registerCommand("/theme", new CommandTheme(),
			i18n( "/theme themefile : Loads the given theme"));
	registerCommand("/topic", new CommandTopic(),
			i18n( "/topic message : Changes the topic of the current channel with the given message"));
	registerCommand("/whois", new CommandWhois(),
			i18n( "/whois nick : Shows whois information of the given nick"));
	registerCommand("/who", new CommandWho(),
			i18n( "/who chan : Shows all users and their user infos on the given channel." ));
	registerCommand("/query", new CommandQuery(),
			i18n( "/query nick : Starts a private conversation with the given nick"));
	registerCommand("/quote", new CommandQuote(),
			i18n("/quote data : Sends server raw data without parsing"));
	registerCommand("/ctcp", new CommandCtcp(),
			i18n("/ctcp target command : Sends the given ctcp command to the specified target"));
	registerCommand("/msg", new CommandMsg(),
			i18n("/msg nick message : Sends the given message to the specified nick"));
	registerCommand("/nick", new CommandNick(),
			i18n("/nick newnick : Changes your nick"));
	registerCommand("/op", new CommandOp(),
			i18n("/op nick : Gives operator privileges to the specified nick"));
	registerCommand("/deop", new CommandDeop(),
			i18n("/deop nick : Takes off operator privileges from the specified nick"));
	registerCommand("/voice", new CommandVoice(),
			i18n("/voice nick : Gives voice mode to the specified nick"));
	registerCommand("/devoice", new CommandDevoice(),
			i18n("/devoice nick : Takes off voice mode from the specified  nick"));
	registerCommand("/server", new CommandServer(),
			i18n("/server <-ssl> host port pass : Changes the current server with the given one"));
	registerCommand("/newserver", new CommandNewServer(),
			i18n("/newserver <-ssl> host port pass : Creates a new server tab and connects to the specified server"));
	registerCommand("/part", new CommandPart(),
			i18n("/part chan <message> : Parts from the specified channel"));
	registerCommand("/ignore", new CommandIgnore(),
			i18n("/ignore mask <-all> <-chan> <-priv> <-notice> <-invite> : Ignores people matching the given mask"));
	registerCommand("/unignore", new CommandUnIgnore(),
			i18n("/unignore mask : Removes the specified ignore mask"));
	registerCommand("/mode", new CommandMode());
	registerCommand("/quit", new CommandQuit(),
			i18n("/quit <message> : Quits the current server"));
	registerCommand("/say", new CommandSay(),
			i18n("/say message : Says the given message"));
	registerCommand("/clear", new CommandClear());
	registerCommand("/clearall", new CommandClearAll());
	registerCommand("/clearchans", new CommandClearAllChans());
	registerCommand("/help", new CommandHelp(), "/help command : Shows the help message for the given command");

#ifdef FEAT_RUBY
	registerCommand("/script", new CommandScript());
#endif
}

bool Command::registerCommand(const QString &c, CommandItem *ci, const QString &helpString)
{
	QString cmd = c.lower();
	if (commands[cmd])
		return false;
	commands.insert(cmd, ci);
	helpStrings[cmd] = helpString;
	QDictIterator<CommandItem> it(commands);
	for( ; it.current(); ++it )
		commandComp.append(it.currentKey());
	return true;
}

bool Command::unregisterCommand(const QString &cmd)
{
	commands.remove(cmd);
	helpStrings.remove(cmd);
	return true;
}

bool Command::nickComplete(ServerElem *se, const QString &str)
{
	QString com = str.section(' ', 0, 0);
	if (com.isEmpty())
		return true;
	if (commands[com])
		return commands[com]->nickComplete(se, str);
	return false;
}

Command *Command::command = NULL;

Command *Command::getInstance()
{
	if (command)
		return command;
	return command = new Command();
}

void Command::userCommand(const QString &command)
{
	getInstance()->realUserCommand(NULL, command);
}

void Command::userCommand(ServerElem *s, const QString &command)
{
	getInstance()->realUserCommand(s, command);
}

void Command::realUserCommand(ServerElem *se, const QString &command)
{
	QString str = command.section(' ', 0, 0);
	str = str.lower();
	CommandItem *ci = commands[str];
	if (ci)
		ci->execute(se, command);
	else
		emit logActive(LOG_RAW, i18n("Commands"),
				i18n("Command not found"), QString::null);
}

void Command::emitLoadTheme(const QString &str)
{
	emit getInstance()->loadTheme(str);
}
void Command::emitOpenQuery(Nick *n)
{
	emit getInstance()->openQuery(n);
}

void Command::log(const QString &arg1, const QString &arg2)
{
	emit getInstance()->logActive(LOG_RAW, arg1, arg2, QString::null);
}

QStringList Command::getCompletion(ServerElem *se, const QString &text)
{
	if (text.find(' ') != -1) {
		QString command = text.section(' ', 0, 0);
		if (!commands[command]) {
			return nullStringList;
		}
		return commands[command]->complete(se, text);
	}
	return commandComp;
}

QStringList Command::computeCompletionStrings(ServerElem *se,
		const QString &text)
{
	QStringList completionStrings;
	if (text.isEmpty() || !text.startsWith("/")) {
		if (text.contains(" ")) {
			completionStrings = nicks;
		} else {
			completionStrings.clear();
			for (QStringList::Iterator it = nicks.begin();
					it != nicks.end(); ++it)
				completionStrings.append(*it + ": ");
		}
		return completionStrings;
	}
	if (nickComplete(se, text)) {
		completionStrings = nicks;
	} else  {
		completionStrings = getCompletion(se, text);
	}
	return completionStrings;
}


void Command::setNicks(const QStringList &nl)
{
	nicks = nl;
}

const QStringList &Command::getNicks()
{
	return nicks;
}

void Command::emitChangeServer(const QString &host, unsigned short port,
		const QString &pass, bool ssl)
{
	emit getInstance()->changeServer(host, port, pass, ssl);
}

void Command::emitNewServer(const QString &host, unsigned short port,
		const QString &pass, bool ssl)
{
	emit getInstance()->newServer(host, port, pass, ssl);
}

void Command::showHelp( const QString &cmd )
{
	if ( !helpStrings.contains( cmd ) ) {
		log( i18n("/help %1").arg( cmd ), i18n("No such command") );
		return;
	}
	QString help = helpStrings[cmd];
	if ( help == QString::null ) {
		log( i18n("/help %1").arg( cmd ), i18n("No help for this command") );
		return;
	}
	log( i18n("/help %1").arg( cmd ), help );
}
