/*
 * $Id: command.h,v 1.14 2004/03/10 16:40:59 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef COMMAND_H
#define COMMAND_H

#include <qstring.h>
#include <qobject.h>
#include <qdict.h>
#include "kwirclog.h"

class ServerElem;
class Server;
class Common;
class Nick;
class Channel;
class CommandItem;

class Command : public QObject
{
	Q_OBJECT
	private:
		Command();
		static Command *command;
		static QDict<CommandItem> commands;
		static QStringList commandComp;
		static QStringList nullStringList;
		static QStringList nicks;
		static QMap<QString, QString> helpStrings;

		void realUserCommand(ServerElem *s, const QString &a);
		void realEmitLoadTheme(const QString &str);
	public:
		static Command *getInstance();

		static void userCommand(ServerElem *se, const QString &a);
		static void userCommand(const QString &a);
		static bool nickComplete(ServerElem *se, const QString &str);
		static QStringList getCompletion(ServerElem *se,
				const QString &text);
		static void emitOpenQuery(Nick *);
		static void emitLoadTheme(const QString &);
		static void emitNewServer(const QString &, unsigned short,
				const QString &, bool);
		static void emitChangeServer(const QString &, unsigned short,
				const QString &, bool);
		static void log(const QString &, const QString &);
		static bool registerCommand(const QString &cmd, CommandItem *ci,
					    const QString &helpString = QString::null);
		static bool unregisterCommand(const QString &cmd);
		static QStringList computeCompletionStrings(ServerElem *,
				const QString &text);
		static const QStringList &getNicks();
		static void showHelp(const QString &cmd);
	signals:
		void openQuery(Nick *);
		void newServer(const QString &, unsigned short,
				const QString &, bool);
		void changeServer(const QString &, unsigned short,
				const QString &, bool);
		void loadTheme(const QString &);
		void logActive(KwircLogType, const QString &, const QString &,
				const QString &);
	public slots:
		void setNicks(const QStringList &);
};

class CommandItem
{
	public:
		virtual ~CommandItem();
		virtual QStringList complete(ServerElem *se, const QString &);
		virtual void execute(ServerElem *se, const QString &) = 0;
		virtual bool nickComplete(ServerElem *se, const QString &);
};

class CommandNickItem : public CommandItem
{
	public:
		virtual ~CommandNickItem();
		virtual bool nickComplete(ServerElem *, const QString &);
};

#endif // COMMAND_H
