/*
 * $Id: ignorelisteditor.ui.h,v 1.2 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

void IgnoreListEditor::init()
{
    QStringList headers = QStringList::split(",", i18n("Mask, Channel, Private, Notice, Invite"));
    ignoreListTable->setColumnLabels(headers);
    ignoreListTable->setColumnStretchable(0, true);
}

void IgnoreListEditor::setIgnoreList( const IgnoreList &l )
{
    int row = 0;
    IgnoreList::Iterator it;
    IgnoreList li = l;

    ignoreListTable->insertRows(0, li.count());

    for (it = li.begin(); it != li.end(); ++it) {
	IgnoreListItem item = it.data();
	ignoreListTable->setText(row, 0, it.key());

	QCheckTableItem * checkItem = new QCheckTableItem(ignoreListTable, "");
	checkItem->setChecked(item.chan);
	ignoreListTable->setItem(row, 1, checkItem);

	checkItem = new QCheckTableItem(ignoreListTable, "");
	checkItem->setChecked(item.priv);
	ignoreListTable->setItem(row, 2, checkItem);

	checkItem = new QCheckTableItem(ignoreListTable, "");
	checkItem->setChecked(item.notice);
	ignoreListTable->setItem(row, 3, checkItem);

	checkItem = new QCheckTableItem(ignoreListTable, "");
	checkItem->setChecked(item.invite);
	ignoreListTable->setItem(row, 4, checkItem);

	ignoreListTable->adjustRow(row);
	ignoreListTable->setRowStretchable(row, false);
	row++;
    }

    ignoreListTable->adjustColumn(1);
    ignoreListTable->adjustColumn(2);
    ignoreListTable->adjustColumn(3);
    ignoreListTable->adjustColumn(4);
}

IgnoreListItem * IgnoreListEditor::getIgnoreListItemFromRow( int r )
{
    bool b[4];
    QTableItem * cell;
    QCheckTableItem * checkItem;

    if (r >= ignoreListTable->numRows()) {
	return NULL;
    }

    for (int i=1; i<=4; i++) {
	cell = ignoreListTable->item(r, i);
	if (cell->rtti() != 2) {
	    // Mayday, the cell is not a QCheckTableItem
	    return NULL;
	}
	checkItem = (QCheckTableItem *) cell;
	b[i-1] = checkItem->isChecked();
    }

    return new IgnoreListItem(b[0], b[1], b[2], b[3]);
}

IgnoreList IgnoreListEditor::getIgnoreList()
{
    IgnoreList l;
    int nbrows = ignoreListTable->numRows();
    for (int i = 0; i < nbrows; i++) {
	QString mask = ignoreListTable->text(i, 0);
	IgnoreListItem * item = getIgnoreListItemFromRow(i);
	if ((item != NULL) && (!mask.isEmpty())) {
	    l[mask] = *item;
	    delete item;
	}
    }

    return l;
}

void IgnoreListEditor::clearAllRows()
{
    QMemArray<int> toBeDeleted(ignoreListTable->numRows());
    for (int i=0; i < ignoreListTable->numRows(); i++) {
	toBeDeleted[i] = i;
    }

    ignoreListTable->removeRows(toBeDeleted);
}

void IgnoreListEditor::removeCurrentRow()
{
    int r = ignoreListTable->currentRow();
    ignoreListTable->removeRow(r);
}

void IgnoreListEditor::addNewRow()
{
    int nbrows = ignoreListTable->numRows();
    ignoreListTable->insertRows(nbrows);
    ignoreListTable->setItem(nbrows, 1, new QCheckTableItem(ignoreListTable, ""));
    ignoreListTable->setItem(nbrows, 2, new QCheckTableItem(ignoreListTable, ""));
    ignoreListTable->setItem(nbrows, 3, new QCheckTableItem(ignoreListTable, ""));
    ignoreListTable->setItem(nbrows, 4, new QCheckTableItem(ignoreListTable, ""));
}
