/*
 * $Id: kextendedsslsocket.h,v 1.6 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 Julien Perrot <blackmore@no-log.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "config.h"

#ifdef HAVE_SSL

#ifndef KEXTENDEDSSLSOCKET_H
#define KEXTENDEDSSLSOCKET_H

#include <openssl/ssl.h>
#include <kextsock.h>
#include <qstring.h>

class KExtendedSSLSocket : public KExtendedSocket
{
	Q_OBJECT
private:
	enum { Idle, Connect, Handshake, Active };
	enum {  TryAgain, Success, Error };

	SSL *ssl;
	X509 *cert;
	BIO *rbio;
	BIO *wbio;
	int mode;

	static bool init_done;
	static SSL_CTX *context;
public:
	KExtendedSSLSocket(const QString& host, int port, int flags = 0);
	virtual ~KExtendedSSLSocket();

	int startHandshake();
	void reset();
	virtual Q_LONG writeBlock (const char *data, Q_ULONG len);
protected:
	virtual unsigned feedReadBuffer (unsigned , const char *, bool = false);
	virtual unsigned feedWriteBuffer (unsigned , const char *);
private:
	void initSSL();
	bool isOutgoingSSLData();
	bool isRecvData();

	void sslReadAll();
	void sslUpdate();
	int doConnect();
	int doHandshake();
private slots:
	void slotSSL_outgoingSSLDataReady();
	void slotSSL_readyRead();
	void slotBytesWritten(int);
	void slot_readyRead();
signals:
	void outgoingSSLDataReady();
	void ssl_handshaken(bool);
	void ssl_readyRead();
};

#endif
#endif
