/*
 * $Id: kwirc.cpp,v 1.83 2004/03/10 16:41:54 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "kwirc.h"
#include "ahistlineedit.h"
#include "kwirctext.h"
#include "server.h"
#include "channel.h"
#include "nick.h"
#include "serverview.h"
#include "servertab.h"
#include "channelview.h"
#include "channeltab.h"
#include "queryview.h"
#include "querytab.h"
#include "nicklistview.h"
#include <kmenubar.h>
#include <qlabel.h>
#include <klocale.h>
#include <kwircpreferences.h>
#include <kiconloader.h>
#include <kprogress.h>
#include <qtooltip.h>
#include "command.h"
#include <kdebug.h>
#include "kwircdebug.h"
#ifdef FEAT_RUBY
#include "rubyscriptmanager.h"
#endif
#include "nickpopupmenu.h"
#include "kaccel.h"
#include <kkeydialog.h>
#include <krootpixmap.h>
#include <dcopclient.h>
#include "themeeditor.h"

#include "serverelem.h"
#include "chanmodeseditor.h"
#include "ignorelisteditor.h"
#include "topicview.h"

Kwirc::Kwirc(QWidget *parent, const char *name)
	 : KDockMainWindow(parent, name),
	 activatedTab(NULL),
	 auxHidden(false),
	 queryTabs(17, false)
{
#ifdef FEAT_RUBY
	RubyScriptManager::init();
	connect(RubyScriptManager::getInstance(),
			SIGNAL(logActive(KwircLogType, const QString &,
					const QString &, const QString &)),
			this, SLOT(slotLogActive(KwircLogType, const QString &,
					const QString &, const QString &)));
#endif
	KConfig * config = KApplication::kApplication()->config();
	KwircPreferences::init(config);

	initActions();
	makeMenu();
	resize(800,600);
	makeGUI();

	Command *cmd = Command::getInstance();
	connect(cmd, SIGNAL(newServer(const QString &, unsigned short,
					const QString&, bool)),
			this, SLOT(newServer(const QString &, unsigned short,
					const QString &, bool)));
	connect(cmd, SIGNAL(changeServer(const QString &, unsigned short,
					const QString&, bool)),
			this, SLOT(changeServer(const QString &,
					unsigned short, const QString &,
					bool)));
	connect(cmd, SIGNAL(openQuery(Nick *)),
			this, SLOT(bindQueryToWindow(Nick *)));
	connect(cmd, SIGNAL(logActive(KwircLogType, const QString &,
					const QString &, const QString &)),
			this, SLOT(slotLogActive(KwircLogType, const QString &,
					const QString &, const QString &)));
	setAutoSaveSettings();
}

Kwirc::~Kwirc()
{
}

void Kwirc::initActions()
{
	quitAction = KStdAction::quit(this, SLOT(close()), actionCollection());
	configureKeysAction = KStdAction::keyBindings(this,
			SLOT(slotConfigureKeys()), actionCollection());
	nextTabAction = new KAction( i18n( "Next tab" ) , Qt::CTRL + Qt::Key_N,
			this, SLOT(activateNextTab()), actionCollection(),
			"NextTab");
	precTabAction = new KAction(i18n("Prec tab"), Qt::CTRL + Qt::Key_P,
			this, SLOT(activatePrecTab()), actionCollection(),
			"PrecTab");
	editSettingsAction = KStdAction::preferences(
			KwircPreferences::getInstance(), SLOT(showSettings()),
			actionCollection());
	closeTabAction = new KAction(i18n("&Close"),
			SmallIconSet("fileclose"), Qt::CTRL + Qt::Key_W,
			this, SLOT(slotCloseActive()), actionCollection());

	popupModeEditorAction = new KAction(i18n("&Edit channel modes"),
			Qt::CTRL + Qt::Key_E, this,
			SLOT(popupModeEditor()), actionCollection());

	popupIgnoreListAction = new KAction(i18n("&Ignore list"),
			Qt::CTRL + Qt::Key_I, this,
			SLOT(popupIgnoreList()), actionCollection());
	popupThemeEditorAction = new KAction(i18n("&Theme Editor"),
			Qt::CTRL + Qt::Key_T, this,
			SLOT(popupThemeEditor()), actionCollection());
	actionCollection()->readShortcutSettings();
}

void Kwirc::slotConfigureKeys()
{
	KKeyDialog::configure(actionCollection());
}

void Kwirc::activateNextTab()
{
	KwircTab *tab = activatedTab->getServerTab()->nextTab(activatedTab);
	if (tab == NULL) {
		serverTabs.find(activatedTab->getServerTab());
		tab = serverTabs.next();
	}
	if (tab == NULL)
		return;
	tab->activate();
}

void Kwirc::activatePrecTab()
{
	KwircTab *tab = activatedTab->getServerTab()->precTab(activatedTab);
	if (tab == NULL) {
		serverTabs.find(activatedTab->getServerTab());
		ServerTab *stab = serverTabs.prev();
		tab = stab;
		if (stab != NULL)
			tab = stab->lastTab();
	}
	if (tab == NULL)
		return;
	tab->activate();
}

void Kwirc::makeMenu()
{
	kwircMenu = new KPopupMenu(this);
	menuBar()->insertItem("&Kwirc", kwircMenu);

	popupModeEditorAction->plug(kwircMenu);
	popupModeEditorAction->setEnabled(false);
	popupIgnoreListAction->plug(kwircMenu);
	popupIgnoreListAction->setEnabled(false);
	popupThemeEditorAction->plug(kwircMenu);

	closeTabAction->plug(kwircMenu);
	kwircMenu->insertSeparator();
	quitAction->plug(kwircMenu);

	serverMenu = new KPopupMenu(this);
	menuBar()->insertItem("&Servers", serverMenu);
	channelMenu = new KPopupMenu(this);
	menuBar()->insertItem("&Channels", channelMenu);
	queryMenu = new KPopupMenu(this);
	menuBar()->insertItem("&Queries", queryMenu);

#ifdef FEAT_RUBY
	menuBar()->insertItem(i18n("S&cripts"),
			RubyScriptManager::getInstance()->popup());
#endif

	settingsMenu = new KPopupMenu(this);
	menuBar()->insertItem(i18n("S&ettings"), settingsMenu);
	configureKeysAction->plug(settingsMenu);
	editSettingsAction->plug(settingsMenu);

	KPopupMenu *help = helpMenu(0, false);
	menuBar()->insertItem(i18n("&Help"), help );
}

void Kwirc::makeGUI()
{
	channelToolbar = toolBar("channels");
	channelToolbar->setText(i18n("Channels and servers"));
	channelToolbar->setIconText(KToolBar::IconTextRight);
	channelToolbar->setFullSize(true);
	connect(channelToolbar, SIGNAL(modechange()),
			this, SLOT(makeToolbar()));
	KwircTab::setCollectionParent(channelToolbar);

	textDock = createDockWidget( "TextDock", QPixmap(), 0l, "text_dock");
	textDock->setDockSite(KDockWidget::DockLeft|KDockWidget::DockRight);
	textDock->setEnableDocking(KDockWidget::DockNone);
	setView(textDock); // central widget in a KDE mainwindow
	setMainDockWidget(textDock); // master dockwidget

	textContainer = new QVBox(textDock);
	textDock->setWidget(textContainer);

	nickListDock = createDockWidget( "Nick list", QPixmap(), 0L,
					"nick_list_dock");
	nickListDock->manualDock(textDock, KDockWidget::DockRight, 80);
	nicklistContainer = new QVBox(nickListDock);
	nickListDock->setWidget(nicklistContainer);

	inputToolbar = toolBar("input");
	inputToolbar->setText(i18n("Chat here!"));
	inputToolbar->setFullSize(true);
	inputToolbar->setBarPos(KToolBar::Bottom);

	input = new aHistLineEdit(inputToolbar);
	inputToolbar->insertWidget(0,50,input);
	inputToolbar->setItemAutoSized (0, true);
	connect(input, SIGNAL(returnPressed()), this,
			SLOT(slotReturnPressed()));
	connect(input, SIGNAL(fillCompletion(const QString &,
					QStringList&)),
			this, SLOT(slotFillCompletion(const QString &,
					QStringList&)));
	connect(KwircPreferences::getInstance(),
		SIGNAL(connectionRequest(ServerSettings *)),
		this, SLOT(newServer(ServerSettings *)));

	connect(Command::getInstance(),
		SIGNAL(loadTheme(const QString&)),
		KwircPreferences::getInstance(),
		SLOT(loadTheme(const QString&)));

	KwircPreferences::autoConnect();
}

void Kwirc::slotReturnPressed()
{
	QString in = input->text();
	if (in.isEmpty()) 	// FIXME
		return;
	input->setText("");
	QStringList lines = QStringList::split( "\n", in );
	// FIX
	for (QStringList::Iterator it = lines.begin();
			it != lines.end(); ++it ) {

		if (activatedTab == NULL
				|| !activatedTab->
				getServerElemServer()->isConnected()) {
			Command::userCommand(*it);
		} else if (activatedTab == NULL) {
			Command::userCommand(*it);
		} else {
			if (activatedTab->getServerElem()) {
				activatedTab->getServerElem()->input(*it);
			} else {
				Server *s =
					activatedTab->getServerElemServer();
				Nick *n = s->nickFromString(
						activatedTab->text());
				n->sendPrivmsg(*it);
				s->deleteNick(n);
			}
		}
	}
}

void Kwirc::slotFillCompletion(const QString &in, QStringList &ret)
{
	if (activatedTab != NULL && activatedTab->getServerElem())
		ret = activatedTab->getServerElem()->complete(in);
}

void Kwirc::setupNewWindow(KwircText *view, KwircTab *tab, ServerElem *elem)
{
	connect(tab, SIGNAL(onTop(KwircTab *)),
			this, SLOT(activate(KwircTab *)));
	connect(view, SIGNAL(notify(unsigned int)),
		tab, SLOT(setNotificationIcon(unsigned int)));

	connectNewServerElem(view, tab, elem);
}

void Kwirc::connectNewServerElem(KwircText *view, KwircTab *tab,
		ServerElem *elem)
{
	connect(elem, SIGNAL(logActive(KwircLogType, const QString&,
					const QString&, const QString&)),
			this, SLOT(slotLogActive(KwircLogType, const QString&,
					const QString&, const QString&)));
	connect(elem, SIGNAL(log(KwircLogType, const QString&,
					const QString&, const QString&)),
			view, SLOT(log(KwircLogType, const QString&,
					const QString&, const QString&)));
	connect(elem, SIGNAL(clearLog()), view, SLOT(clear()));
	connect(elem, SIGNAL(stringIdChange(const QString&)),
			tab, SLOT(setText(const QString &)));
}

void Kwirc::slotLogActive(KwircLogType type, const QString &name,
		const QString &arg1, const QString &arg2)
{
	if (activatedTab == NULL)
		return;
	KwircText *kt = activatedTab->getMainView()->getKwircText();
	if (kt)
		kt->log(type, name, arg1, arg2);
}

void Kwirc::newServer(ServerSettings *settings)
{
	Server *server = new Server(
		settings->hostName(),
		settings->port(),
		settings->label(),
		settings->userName(),
		settings->password(),
		settings->useSSL(),
		settings->nickName(),
		settings->currentGroup(),
		settings->ignoreList()
	);
	server->setAutoJoin(settings->autojoin(), settings->autojoinKeys());
	bindServerToWindow(server);
	server->startConnect();
}

void Kwirc::newServer(const QString &host, unsigned short port,
		const QString &pass, bool ssl)
{
	Server *server = new Server(
		host,
		port,
		host,
		KwircSettings::userName(),
		pass,
		ssl,
		KwircSettings::nickName(),
		QString::null,
		QStringList() );
	bindServerToWindow(server);
	server->startConnect();
}

void Kwirc::changeServer(const QString &server, unsigned short port,
		const QString &pass, bool ssl)
{
	if (!activatedTab)
		return;
	Server *s = activatedTab->getServerElemServer();
	s->changeServer(server, port, QString::null,
			pass, ssl, QString::null);
	s->startConnect();
}

void Kwirc::bindServerToWindow(Server *server)
{
	ServerView *serverView = new ServerView(textContainer);
	ServerTab *tab = new ServerTab(server, server->getStringId());
	setupNewWindow(serverView->getKwircText(), tab, server);
	tab->setMainView(serverView);

	servers.append(server);
	serverTabs.append(tab);
	makeToolbar();
	connect(server, SIGNAL(newChannel(Channel *)),
			this, SLOT(bindChannelToWindow(Channel *)));
	connect(server, SIGNAL(newQuery(Nick *)),
			this, SLOT(bindQueryToWindow(Nick *)));

	connect(tab, SIGNAL(tabClose()),
		this, SLOT(removeServer()));

	KProgress *lagometer = new KProgress(Server::MAX_LAG, inputToolbar);
	lagometer->setFormat("%vs");
	lagometer->setMaximumWidth(50);
	QToolTip::add(lagometer, server->getStringId());
	inputToolbar->insertWidget(0,5,lagometer);
	connect(server, SIGNAL(lag(int)),
		lagometer, SLOT(setValue(int)));

	tab->lowerViews();
	if (activatedTab == NULL) {
		activatedTab = tab;
		tab->activate();
	} else if (KwircSettings::activateNew()) {
		tab->activate();
	}
}

ServerTab *Kwirc::getServerTab(Server *server)
{
	int index = servers.find(server);
	if ( index == -1 ) {
		kdError() << k_funcinfo << "server not found in servers" << endl;
		return NULL;
	}
	return serverTabs.at(index);
}

Server *Kwirc::getServer(ServerTab *st)
{
	int index = serverTabs.find(st);
	if ( index == -1 ) {
		kdError() << k_funcinfo << "st not found in serverTabs" << endl;
		return NULL;
	}
	return servers.at(index);
}

void Kwirc::bindChannelToWindow(Channel *c)
{
	ServerTab *st =  getServerTab(c->getServer());
	if ( !st ) {
		kdError() << k_funcinfo << "No ServerTab for : " << c->getServer() << endl;
		return;
	}

	ChannelTab *tab = st->channelTab(c->getStringId());

	if (tab && tab->getServerElem()) {
		// - tab already open and connected to the same nick
		return;
	}

	ChannelView *channelView;
	NicklistView *nicklistView;
	NickPopupMenu *nickPopup;

	if (tab == NULL) {
		channelView = new ChannelView(textContainer);
		tab = new ChannelTab(c, c->getStringId(), st);
		tab->setMainView(channelView);
		st->addChannelTab(tab);
		makeToolbar();

		connect(tab, SIGNAL(tabPreClose()), c, SLOT(part()));
		connect(tab, SIGNAL(tabClose()),
				this, SLOT(removeChannel()));

		nickPopup = new NickPopupMenu();
		channelView->getKwircText()->setNickPopup(nickPopup);

		connect(channelView->getKwircText(),
				SIGNAL(openQuery(const QString &)),
				c->getServer(),
				SLOT(slotQuery(const QString &)));


		nicklistView = new NicklistView(nicklistContainer);
		nicklistView->setNickPopup(nickPopup);

		tab->setAuxView(nicklistView);
		connect(nicklistView, SIGNAL(triggerQuery(const QString &)),
				c->getServer(),
				SLOT(slotQuery(const QString &)));
	} else {
		channelView = (ChannelView *) tab->getMainView();
		nicklistView = (NicklistView *) tab->getAuxView();
		tab->setServerElem(c);
		nickPopup = channelView->getKwircText()->getNickPopup();
	}
	connect(nickPopup, SIGNAL(whois(const QString &)),
		c, SLOT(whois(const QString &)));
	connect(nickPopup, SIGNAL(op(const QString &)),
		c, SLOT(op(const QString &)));
	connect(nickPopup, SIGNAL(deop(const QString &)),
		c, SLOT(deop(const QString &)));
	connect(nickPopup, SIGNAL(voice(const QString &)),
		c, SLOT(voice(const QString &)));
	connect(nickPopup, SIGNAL(devoice(const QString &)),
		c, SLOT(devoice(const QString &)));
	connect(nickPopup, SIGNAL(ctcp(const QString &, const QString &)),
		c, SLOT(ctcp(const QString &, const QString &)));
	connect(nickPopup, SIGNAL(triggerQuery(const QString &)),
		c->getServer(), SLOT(slotQuery(const QString &)));

	setupNewWindow(channelView->getKwircText(), tab, c);

	connect(c, SIGNAL(preDelete()), tab, SLOT(deletedServerElem()));

	connect(c, SIGNAL(newTopic(const QString &)),
		channelView->getTopicView(), SLOT(setTopic(const QString &)));

	connect(c, SIGNAL(left()), tab, SLOT(close()));

	connect(channelView->getTopicView(),
			SIGNAL(topicChanged(const QString &)),
			c, SLOT(topic(const QString &)));

	connect(c, SIGNAL(channelUsersChange(const QPtrList<Nick> &,
					const QPtrList<Nick> &,
					const QPtrList<Nick> &)),
			nicklistView,
			SLOT(setChannelUsers(const QPtrList<Nick> &,
					const QPtrList<Nick> &,
					const QPtrList<Nick> &)));
	tab->lowerViews();
	if (KwircSettings::activateNew())
		tab->activate();
}

void Kwirc::bindQueryToWindow(Nick *nick)
{
	ServerTab *st =  getServerTab(nick->getServer());
	QueryTab *qt = st->queryTab(nick->getStringId());
	if (qt && qt->getServerElem()) {
		// - tab already open and connected to the same nick
		return;
	}

	QueryView *queryView;

	if (qt == NULL) {
		queryView = new QueryView(textContainer);
		qt = new QueryTab(nick, nick->getStringId(), st);
		qt->setMainView(queryView);
		st->addQueryTab(qt);
		connect(qt, SIGNAL(tabClose()), this, SLOT(removeQuery()));
		makeToolbar();
	} else {
		queryView = (QueryView *)qt->getMainView();
		qt->setServerElem(nick);
	}

	setupNewWindow(queryView->getKwircText(), qt, nick);

	connect(nick, SIGNAL(preDelete()), qt, SLOT(deletedServerElem()));

	qt->lowerViews();
	if (KwircSettings::activateNew())
		qt->activate();
}

void Kwirc::activate(KwircTab *kt)
{
	if (activatedTab != NULL) {
		activatedTab->lowerViews();

		if (activatedTab->getAuxView()) {
			disconnect(activatedTab->getAuxView(),
					SIGNAL(completionListChange(
							const QStringList&)),
					Command::getInstance(), SLOT(setNicks(
							const QStringList &)));
		}
	}

	activatedTab = kt;
	kt->setResetIcon();

	if ( kt->getServerElem() ) {
		popupIgnoreListAction->setEnabled(true );
	}

	if (kt->getServerElem() && kt->getServerElem()->getType()
				== ServerElem::TYPE_CHANNEL) {
		popupModeEditorAction->setEnabled(true);
	} else {
		popupModeEditorAction->setEnabled(false);
	}

	if (activatedTab->hasAuxView()) {
		if (auxHidden) {
			nickListDock->dockBack();
			nickListDock->show();
			auxHidden = false;
		}
		Command::getInstance()->setNicks(
				((NicklistView*)activatedTab->getAuxView())
				->getCompletionList());
		input->resetCompletion();
		connect(activatedTab->getAuxView(),
				SIGNAL(completionListChange(
						const QStringList&)),
				Command::getInstance(),
				SLOT(setNicks(const QStringList &)));
	} else {
		if (!auxHidden) {
			nickListDock->undock();
			nickListDock->hide();
			auxHidden = true;
		}
	}
	activatedTab->raiseViews();
}

void Kwirc::makeToolbar()
{
	serverMenu->clear();
	channelMenu->clear();
	queryMenu->clear();

	channelToolbar->clear();
	ServerTab *tab;
	for (tab = serverTabs.first(); tab; tab = serverTabs.next()) {
		tab->plug(channelToolbar);
		tab->plug(serverMenu);

		queryMenu->insertTitle(SmallIcon("server"), tab->text());
		channelMenu->insertTitle(SmallIcon("server"), tab->text());

		ChannelTab *ct;
		QPtrList<ChannelTab> clist = tab->getChannelTabs();
		for (ct = clist.first(); ct; ct = clist.next()) {
			ct->plug(channelToolbar);
			ct->plug(channelMenu);
		}

		QueryTab *qt;
		QPtrList<QueryTab> qlist = tab->getQueryTabs();
		for (qt = qlist.first(); qt; qt = qlist.next()) {
			qt->plug(channelToolbar);
			qt->plug(queryMenu);
		}

		channelToolbar->insertSeparator();
		queryMenu->insertSeparator();
		channelMenu->insertSeparator();
	}
}

void Kwirc::quitAllServers(const QString &str)
{
	Server *server;
	for (server = servers.first(); server; server = servers.next()) {
		server->sendQuit(str);
	}
}

bool Kwirc::queryClose()
{
	KwircSettings::writeConfig();
	quitAllServers(KwircSettings::quitMessage());
	return true;
}

void Kwirc::slotCloseActive()
{
	if (activatedTab == NULL)
		return;
	activatedTab->close();
}

// FIXME completly buggy
// forget about this server: we should close all channel/query windows
// then send /QUIT
void Kwirc::removeServer()
{
	ServerTab *tab = (ServerTab *)sender();
	Server * server = getServer(tab);
	if (!server)
		return;

	//server->sendQuit(quitMsg);

	serverTabs.remove(tab);
	servers.remove(server);

	// TODO: delete serverView, tab, server
	makeToolbar();

	if (serverTabs.count() > 0) {
		activate(serverTabs.at(0));
	}
}

/* close window */
void Kwirc::removeChannel()
{
	ChannelTab *tab = (ChannelTab *)sender();
	ServerTab *servTab = tab->getServerTab();
	KwircTab *kt = servTab->precTab(tab);
	servTab->removeChannelTab(tab);
	delete tab->getMainView()->getKwircText()->getNickPopup();
	delete tab->getMainView();
	delete tab->getAuxView();
	delete tab;
	activatedTab = NULL;
	kt->activate();
	makeToolbar();
}

void Kwirc::removeQuery()
{
	QueryTab *tab = (QueryTab *)sender();
	disconnect(tab->getServerElem());
	ServerTab * servTab = tab->getServerTab();
	KwircTab *kt = servTab->precTab(tab);
	servTab->removeQueryTab(tab);
	delete tab->getMainView();
	delete tab;
	activatedTab = NULL;
	kt->activate();
	makeToolbar();
}

void Kwirc::popupModeEditor()
{
	ServerElem *se = activatedTab->getServerElem();
	if (se->getType() != ServerElem::TYPE_CHANNEL)
		return;
	Channel *c = (Channel *)se;
	ChanModesEditor *editor =
		new ChanModesEditor(0, i18n("Channel modes editor").latin1(),
				true);
	editor->setChannel(c);
	if (editor->exec() == QDialog::Accepted ) {
		QString modes = editor->getChangedModes();
		QStringList masks = editor->getBanMasks();
		if (!modes.isEmpty())
			c->modes(modes, "");
		c->setBanMasks(masks);

		if (editor->hasKey()) {
			c->setKey(editor->getKey());
		} else {
			c->unsetKey();
		}

		if (editor->hasLimit()) {
			c->setLimit(editor->getLimit());
		} else {
			c->unsetLimit();
		}

	}
	delete editor;
}

void Kwirc::popupIgnoreList()
{
	Server * server = activatedTab->getServerElem()->getServer();
	if ( !server ) {
		kdWarning() << k_funcinfo << "No server" << endl;
		return;
	}

	IgnoreList l = server->getIgnoreList();
	IgnoreList::Iterator it;

	IgnoreListEditor *editor =
		new IgnoreListEditor(0, i18n("Ignore List editor").latin1(),
				true);
	editor->setIgnoreList(l);
	if (editor->exec() == QDialog::Accepted ) {
		server->setIgnoreList(editor->getIgnoreList());
		server->saveIgnoreList();
	}
	delete editor;
}

void Kwirc::popupThemeEditor()
{
	ThemeEditor *editor = new ThemeEditor(0, i18n("Theme editor").latin1(), false);
	editor->exec();
	KwircPreferences::getInstance()->getCurrentTheme()->refresh();
	delete editor;
}
