/*
 * $Id: kwirc.h,v 1.22 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef KWIRC_H
#define KWIRC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <kdockwidget.h>
#include <ktoolbar.h>
#include <kpopupmenu.h>
#include <qhbox.h>
#include <qptrlist.h>
#include <qsignalmapper.h>
#include "kwirclog.h"
#include "kwircpreferences.h"
#include <qdict.h>

class aHistLineEdit;
class KwircText;
class Server;
class ServerView;
class ServerTab;
class Channel;
class Server;
class Nick;
class KwircTab;
class ChannelTab;
class QueryTab;
class ServerElem;
class Nick;
#include <qvbox.h>

/** Kwirc is the base class of the project */
class Kwirc : public KDockMainWindow
{
	Q_OBJECT
	private:
		KPopupMenu *kwircMenu;
		KPopupMenu *serverMenu;
		KPopupMenu *channelMenu;
		KPopupMenu *queryMenu;
		KPopupMenu *settingsMenu;
#ifdef FEAT_RUBY
		KPopupMenu *scriptMenu;
#endif
		KToolBar *channelToolbar;
		KToolBar *inputToolbar;
		KDockWidget *textDock;
		KDockWidget *nickListDock;
		aHistLineEdit *input;
		KwircTab *activatedTab;
		bool auxHidden;

		QPtrList<ServerTab> serverTabs;
		QPtrList<Server> servers;
		QDict<QueryTab> queryTabs;

		QVBox *textContainer;
		QVBox *nicklistContainer;

		KAction *quitAction;
		KAction *configureKeysAction;
		KAction *nextTabAction;
		KAction *precTabAction;
		KAction *editSettingsAction;
		KAction *closeTabAction;
		KAction *popupModeEditorAction;
		KAction *popupIgnoreListAction;
	KAction *popupThemeEditorAction;

		void makeGUI();
		void makeMenu();
		void setupNewWindow(KwircText *view, KwircTab *tab,
				ServerElem *elem);
		void connectNewServerElem(KwircText *view, KwircTab *tab,
				ServerElem *elem);
		ServerTab *getServerTab(Server *server);
		Server *getServer(ServerTab *tab);
		void initActions();
		void quitAllServers(const QString &str);
	private slots:
		void makeToolbar();
	public:
		Kwirc(QWidget* parent = 0, const char *name = 0);
		~Kwirc();

	protected:
		bool queryClose();
	public slots:
		void newServer(ServerSettings *);
		void bindServerToWindow(Server *);
		void bindChannelToWindow(Channel *);
		void bindQueryToWindow(Nick *);
		void activate(KwircTab *);
		void slotReturnPressed();
		void slotFillCompletion(const QString &in, QStringList &ret);
		void slotLogActive(KwircLogType, const QString&,
				const QString&, const QString&);
		void slotCloseActive();
		void removeServer();
		void removeChannel();
		void removeQuery();
		void activatePrecTab();
		void activateNextTab();
		void slotConfigureKeys();
		void popupModeEditor();
		void popupIgnoreList();
		void popupThemeEditor();
		void newServer(const QString &server, unsigned short port,
			const QString &pass, bool ssl);
		void changeServer(const QString &server, unsigned short port,
			const QString &pass, bool ssl);
};

#endif
