/*
 * $Id: kwircpreferences.cpp,v 1.54 2004/03/10 16:49:03 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "kwircpreferences.h"
#include "kwirctheme.h"
#include "kwirctab.h"
#include <qregexp.h>
#include <qimage.h>
#include <kiconloader.h>

#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include "server.h"
#include <qpixmap.h>
#include <qstylesheet.h>	// escape
#include <kstandarddirs.h>
#include <kglobal.h>
#include <qfile.h>
#include <kapplication.h>
#include <klocale.h>
#include <kglobalsettings.h>
#include <kcmdlineargs.h>

#include "misc_settings.h"
#include "servermanager.h"
#include "appearance.h"
#include "kdebug.h"

bool KwircPreferences::beep;
int KwircPreferences::nickPad = 9;
QString KwircPreferences::version;
QString KwircPreferences::clientinfo;
QString KwircPreferences::userinfo;
QString KwircPreferences::notIcons[5];
QPixmap *KwircPreferences::opPixmap;
QPixmap *KwircPreferences::voicePixmap;
QPixmap *KwircPreferences::stdPixmap;
QPixmap *KwircPreferences::opAwayPixmap;
QPixmap *KwircPreferences::voiceAwayPixmap;
QPixmap *KwircPreferences::stdAwayPixmap;
QString KwircPreferences::hostname;
KwircTheme *KwircPreferences::currentTheme = NULL;
QPtrList<ServerSettings> KwircPreferences::connect_servers;

KwircPreferences *KwircPreferences::prefs = NULL;

KwircPreferences *KwircPreferences::getInstance()
{
	if (prefs == NULL)
		prefs = new KwircPreferences();
	return prefs;
}

KwircPreferences::KwircPreferences()
	: QObject()
{
}

KwircPreferences::~KwircPreferences()
{
}

bool KwircPreferences::isBeepEnabled()
{
	return beep;
}

void KwircPreferences::beepEnable(bool yes)
{
	beep = yes;
}

void KwircPreferences::init(KConfig * config)
{
	nickPad = 9;

	//FIXME
	/*
	notIcons[KwircTab::NOT_RESET]      = QString("openterm");
	notIcons[KwircTab::NOT_RESET_SERV] = QString("network");
	notIcons[KwircTab::NOT_WALK]       = QString("goto");
	notIcons[KwircTab::NOT_TALK]       = QString("bookmark");
	notIcons[KwircTab::NOT_HL]         = QString("idea");
	*/



	opPixmap = new QPixmap(locate("data", "kwirc/icons/op.png"));
	voicePixmap = new QPixmap(locate("data", "kwirc/icons/voice.png"));
	stdPixmap = new QPixmap(locate("data", "kwirc/icons/standard.png"));

	opAwayPixmap = new QPixmap(locate("data", "kwirc/icons/op_away.png"));
	voiceAwayPixmap = new QPixmap(locate("data",
				"kwirc/icons/voice_away.png"));
	stdAwayPixmap = new QPixmap(locate("data",
				"kwirc/icons/standard_away.png"));

	addSmiley("basic_smile", "icon_smile.png");
	addSmiley("big_smile", "icon_biggrin.png");
	addSmiley("confused", "icon_confused.png");
	addSmiley("sunglasses", "icon_cool.png");
	addSmiley("cry", "icon_cry.png");
	addSmiley("bigeyes", "icon_eek.png");
	addSmiley("lol", "icon_razz.png");
	addSmiley("wink", "icon_wink.png");
	addSmiley("ass", "icon_ass.png");
	addSmiley("dick", "icon_dick.png");
	addSmiley("borne", "icon_borne.png");
	addSmiley("turtle", "icon_turtle.png");

	/*
	if (!config->hasGroup("MiscSettings")) {
		createFirstConfig();
		firstRun = true;
	}
	*/

	getInstance()->loadSettings();



	if (!config) {
		//Should not happen
		return;
	}

	hostname = config->readEntry("hostname", QString("dtc.com"));
	version = config->readEntry("version",
		QString("kwirc 0.1-pre (http://kwirc.sf.net) $Revision: 1.54 $"));
	clientinfo = config->readEntry("clientinfo",
			QString("Sexy and willingful"));
	userinfo = config->readEntry("userinfo",
			QString("I must be sexy caus' I am using Kwirc!"));

	//if (firstRun)
	//	getInstance()->showSettings();
}

void KwircPreferences::autoConnect()
{
	getInstance()->doAutoConnect();
}

void KwircPreferences::doAutoConnect()
{
	ServerSettings *settings;
	for ( settings = connect_servers.first(); settings; settings = connect_servers.next() ) {
		emit connectionRequest(settings);
	}
}

const QString &KwircPreferences::getClientinfoReply()
{
	return clientinfo;
}

const QString &KwircPreferences::getVersionReply()
{
	return version;
}

//FIXME nick is not updated
bool KwircPreferences::isToHighlight(const QString &s)
{
	QString nick = KwircSettings::nickName();
	if(!nick.isEmpty() && s.contains(QRegExp(QString("\\b%1\\b")
					.arg(QRegExp::escape(nick)))) >= 1)
		return true;
	QString value = KwircSettings::hilightWords();
	if (KwircSettings::hilightOnRegExp()) {
		QRegExp regExp = QRegExp(value);
		if (s.contains(regExp))
			return true;
		return false;
	} else {
		value.replace(QRegExp("\\s"), "");
		QStringList words = QStringList::split(",", value);
		for (QStringList::Iterator it = words.begin();
				it != words.end();
				++it ) {
			if(s.contains(QRegExp(QString("\\b%1\\b")
					.arg(QRegExp::escape(*it)))) >= 1)
				return true;
		}
		return false;
	}
}

int KwircPreferences::getNickPad()
{
	return nickPad;
}

/*
void KwircPreferences::nickChange(const QString &, const QString &n)
{
	nick = n;
}*/

const QString KwircPreferences::getNotIcon(unsigned int level)
{
	switch ( level ) {
	case KwircTab::NOT_RESET:
		return KwircSettings::defaultIcon();
		break;
	case KwircTab::NOT_RESET_SERV:
		return KwircSettings::defaultServerIcon();
		break;
	case KwircTab::NOT_WALK:
		return KwircSettings::walkIcon();
		break;
	case KwircTab::NOT_TALK:
		return KwircSettings::talkIcon();
		break;
	case KwircTab::NOT_HL:
		return KwircSettings::hilightIcon();
		break;
	default:
		return "filenew";
	}
}

const QPixmap &KwircPreferences::getOpPixmap()
{
	return *opPixmap;
}

const QPixmap &KwircPreferences::getVoicePixmap()
{
	return *voicePixmap;
}

const QPixmap &KwircPreferences::getStdPixmap()
{
	return *stdPixmap;
}

const QPixmap &KwircPreferences::getOpAwayPixmap()
{
	return *opAwayPixmap;
}

const QPixmap &KwircPreferences::getVoiceAwayPixmap()
{
	return *voiceAwayPixmap;
}

const QPixmap &KwircPreferences::getStdAwayPixmap()
{
	return *stdAwayPixmap;
}

KwircTheme * KwircPreferences::getCurrentTheme()
{
	return currentTheme;
}

const QString &KwircPreferences::getHostname()
{
	return hostname;
}

void KwircPreferences::loadTheme(const QString &path)
{
	QString fullPath;
	if (QFile::exists(path)) {
		fullPath = path;
	} else {
		QString p = "kwirc/themes/" + path;
		p.replace(QRegExp("//+" ), "/");
		fullPath = KGlobal::dirs()->findResource("data", p);
	}

	if (fullPath == QString::null ) {
		kdError() << k_funcinfo << "Theme " << path << " not found !" << endl;
	}

	if (currentTheme) {
		currentTheme->loadTheme(fullPath);
	} else {
		currentTheme = new KwircTheme(fullPath);
	}
}

void KwircPreferences::addSmiley(const QString &key, const QString &file)
{
	QStringList smileyDirs = KGlobal::dirs()->findDirs("data",
			"kwirc/smileys/");
	for (QStringList::Iterator it = smileyDirs.begin(); it != smileyDirs.end(); ++it) {
		QString path = *it + "/" + file;
		path.replace(QRegExp("//+" ), "/");
		if (QFile::exists(path)) {
			QMimeSourceFactory::defaultFactory()->setImage(key,
				QImage(path));
			break;
		}
	}
}

void KwircPreferences::loadSettings()
{
	prefs->loadTheme(KwircSettings::themeName());

	emit bgColorChange();
	emit textColorChange();

	QStringList serversList = KwircSettings::connectOrder();

	if ( !KCmdLineArgs::parsedArgs()->isSet("autoconnect") )
		return;

	for (QStringList::Iterator it = serversList.begin(); it != serversList.end(); ++it ) {
		ServerSettings * settings = new ServerSettings(*it);
		settings->readConfig();
		if (settings && settings->autoConnect()) {
			connect_servers.append(settings);
		}
	}
}

void KwircPreferences::showSettings()
{
	if(KConfigDialog::showDialog("settings"))
		return;
	KNotify::KNotifyWidget * notifyWidget = new KNotify::KNotifyWidget();
	KConfigDialog * dialog = new KConfigDialog( 0,  "settings", KwircSettings::self(),
						    KDialogBase::IconList );

	dialog->addPage(new MiscSettings(0, "MiscSettings"), "MiscSettings",
			      "package_settings", i18n("Misc settings"));
	AppearanceSettings * appearance = new AppearanceSettings(0, "Appearance" );
	dialog->addPage(appearance, "Appearance", "background",
			i18n("Global appearance settings"));
	ServerManager *manager = new ServerManager(0, "ServerManager");
	dialog->addPage(manager, "Servers", "package_network",
				      i18n("Servers"), false);
	notifyWidget->clear();
	KNotify::Application* app = notifyWidget->addApplicationEvents( "kwirc/eventsrc" );
	if ( app ) {
		notifyWidget->addVisibleApp(app);
		dialog->addPage(notifyWidget, "Notifications", "knotify", i18n( "Notifications" ), false);
	}

	connect( manager,  SIGNAL( changed(bool) ), getInstance(), SLOT(settingsChangedSlot(bool)));
	connect( notifyWidget,  SIGNAL(changed( bool ) ), getInstance(), SLOT(settingsChangedSlot(bool)));
	connect( appearance,  SIGNAL(changed( bool ) ),  getInstance(), SLOT(settingsChangedSlot(bool)));

	connect(manager, SIGNAL(connectionRequest(ServerSettings *)),
		getInstance(), SLOT(handleConnectionRequest(ServerSettings *)));

	connect(dialog, SIGNAL(okClicked()), manager, SLOT(saveSettings()));
	connect(dialog, SIGNAL(applyClicked()), manager, SLOT(saveSettings()));
	connect(dialog, SIGNAL(okClicked()), notifyWidget, SLOT(save()));
	connect(dialog, SIGNAL(applyClicked()), notifyWidget, SLOT(save()));
	connect(dialog, SIGNAL(okClicked()), appearance, SLOT(save()));
	connect(dialog, SIGNAL(applyClicked()), appearance, SLOT(save()));

	connect(dialog, SIGNAL(okClicked()), getInstance(), SLOT(saveSettings()));
	connect(dialog, SIGNAL(applyClicked()), getInstance(), SLOT(saveSettings()) );
	connect(dialog, SIGNAL(settingsChanged()), getInstance(), SLOT(loadSettings()));

	dialog->show();
}

void KwircPreferences::handleConnectionRequest(ServerSettings *settings)
{
	emit connectionRequest(settings);
}


void KwircPreferences::handleConnectionRequest(const QString &group)
{
	handleConnectionRequest( new ServerSettings(group ) );
}

void KwircPreferences::saveSettings()
{
	if ( KConfigDialog * dialog = KConfigDialog::exists ( "settings" ) ) {
		dialog->enableButtonApply( false );
	}
}

void KwircPreferences::settingsChangedSlot(bool b)
{
	if ( KConfigDialog * dialog = KConfigDialog::exists ( "settings"  ) ) {
		dialog->enableButtonApply( b );
	}
}
