/*
 * $Id: kwircpreferences.h,v 1.31 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#ifndef KWIRCPREFERENCES_H
#define KWIRCPREFERENCES_H

#include <qiconset.h>
#include <kconfig.h>
#include <qptrlist.h>
#include <qobject.h>
#include <qpixmap.h>
#include <knotifydialog.h>
#include "kwirctheme.h"
#include "kwircsettings.h"
#include "serversettings.h"
#include <kconfigdialog.h>

namespace KNotify {
  class KNotifyWidget;
}

class KwircPreferences : public QObject
{
	Q_OBJECT
private:
	int maxServerNumber;

	static KwircPreferences *prefs;
	static bool beep;
	static QString notIcons[5];
	static QPixmap *opPixmap, *voicePixmap, *stdPixmap;
	static QPixmap *opAwayPixmap, *voiceAwayPixmap, *stdAwayPixmap;
	static QString nick;
	static QString hostname;
	static int nickPad;
	static QString version;
	static QString clientinfo;
	static QString userinfo;

	static KwircTheme *currentTheme;

	static QPtrList<ServerSettings> connect_servers;
	void doAutoConnect();
	static void addSmiley(const QString &, const QString &);

	KwircPreferences();
	~KwircPreferences();
public:
	static void autoConnect();

	static KwircPreferences *getInstance();
	static void init(KConfig *);

	static bool isBeepEnabled();
	static void beepEnable(bool yes);

	static const QString getNotIcon(unsigned int);

	static const QPixmap &getOpPixmap();
	static const QPixmap &getVoicePixmap();
	static const QPixmap &getStdPixmap();
	static const QPixmap &getOpAwayPixmap();
	static const QPixmap &getVoiceAwayPixmap();
	static const QPixmap &getStdAwayPixmap();
	static const QString &getHostname();

	static const QString &getClientinfoReply();
	static int getNickPad();
	static bool isToHighlight(const QString &);
	static const QString &getVersionReply();

	// Appearance settings getter
	static KwircTheme *getCurrentTheme();
public slots:
	void loadTheme(const QString &);
	void loadSettings();
	void showSettings();
private slots:
	void handleConnectionRequest(const QString &);
	void handleConnectionRequest(ServerSettings * );
	void saveSettings();
	void settingsChangedSlot(bool);
signals:
	void connectionRequest(ServerSettings *);
	void textColorChange();
	void bgColorChange();
	void transparencyChange();
};

#endif
