/*
 * $Id: kwirctab.cpp,v 1.18 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "kwirctab.h"
#include "kwirctext.h"
#include "kwircpreferences.h"
#include "kwircsettings.h"
#include "serverelem.h"
#include "kwirctabview.h"

const QString KwircTab::KWIRC_TAB_GROUP = QString("kwirc_tab_group");
QObject *KwircTab::collectionParent = NULL;
KActionCollection *KwircTab::parent = NULL;

void KwircTab::toDisplay(QString &str)
{
	unsigned int maxTabLength = KwircSettings::maxTabLength();
	if (str.length() > maxTabLength
			&& maxTabLength > 2) {
		str.truncate(maxTabLength - 2);
		str.append("..");
	}
	str.replace("&", "&&");
}

KwircTab::KwircTab(ServerElem *se, const QString& text,
		const QString& iconName, const KShortcut &sc)
	 :KRadioAction(text, iconName, sc, getParent(), text.latin1())
{
	QString disp = text;
	toDisplay(disp);
	setText(disp);
	serverElem = se;
	serverElemServer = se->getServer();
	mainView = NULL;
	auxView = NULL;
	setExclusiveGroup(KWIRC_TAB_GROUP);
	connect(this, SIGNAL(activated()), this, SLOT(emitOnTop()));
	setResetIcon();
}

KwircTab::~KwircTab()
{
}

void KwircTab::setCollectionParent(QObject*parent)
{
	collectionParent = parent;
}

KActionCollection *KwircTab::getParent()
{
	if (parent == NULL)
		return parent = new KActionCollection(collectionParent);
	return parent;
}

void KwircTab::setMainView(KwircTabView *kt)
{
	mainView = kt;
}

bool KwircTab::hasAuxView()
{
	return auxView != NULL;
}

KwircTabView *KwircTab::getMainView()
{
	return mainView;
}

void KwircTab::lowerViews()
{
	mainView->hide();
	if (auxView) {
		auxView->hide();
	}
}

void KwircTab::raiseViews()
{
	mainView->show();
	if (auxView) {
		auxView->show();
	}
}

void KwircTab::setAuxView(QWidget *widget)
{
	auxView = widget;
}

QWidget *KwircTab::getAuxView()
{
	return auxView;
}

void KwircTab::emitOnTop()
{
	emit onTop(this);
}

void KwircTab::setNotificationIcon(unsigned int level)
{
	if ((level > currentLevel) && !isChecked()) {
		currentLevel = level;
	}
	setIcon(KwircPreferences::getNotIcon(currentLevel));
}

void KwircTab::setResetIcon()
{
	currentLevel = NOT_RESET;
	setIcon(KwircPreferences::getNotIcon(NOT_RESET));
}

void KwircTab::close()
{
	emit tabPreClose();
	emit tabClose();
}

ServerElem *KwircTab::getServerElem()
{
	return serverElem;
}

Server *KwircTab::getServerElemServer()
{
	return serverElemServer;
}

void KwircTab::setServerElem(ServerElem *se)
{
	serverElem = se;
	serverElemServer = se->getServer();
}

void KwircTab::deletedServerElem()
{
	serverElem = NULL;
}

#include "kwirctab.moc"
