/*
 * $Id: kwirctext.cpp,v 1.63 2004/03/09 14:46:31 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "kwirctext.h"
#include <qfont.h>
#include <qstylesheet.h>
#include <kglobalsettings.h>
#include "kwircpreferences.h"
#include <qregexp.h>
#include <ktextbrowser.h>
#include "kwirclog.h"
#include "kwirctheme.h"
#include "kwirctab.h"
#include <kapplication.h>
#include <kurl.h>
#include <kdebug.h>
#include "kwircdebug.h"
#include <qtextedit.h>
#include <qpainter.h>
#include <qimage.h>
#include <klocale.h>
#include <knotifyclient.h>
#include "kwircsettings.h"
#include <qdatetime.h>

const char* KwircText::colorCodes[16]={"ffffff","000000","000080","008000",
	"ff0000","a52a2a","800080","ff8000", "808000","00ff00","008080",
	"00ffff", "0000ff","ffc0cb","a0a0a0","c0c0c0"};

QRegExp KwircText::basic_smile_re;
QRegExp KwircText::big_smile_re;
QRegExp KwircText::confused_re;
QRegExp KwircText::sunglasses_re;
QRegExp KwircText::cry_re;
QRegExp KwircText::big_eyes_re;
QRegExp KwircText::lol_re;
QRegExp KwircText::wink_re;
QRegExp KwircText::ass_re;
QRegExp KwircText::dick_re;
QRegExp KwircText::borne_re;
QRegExp KwircText::turtle_re;

KwircText::KwircText(QWidget *parent, const char *name):
	KTextBrowser(parent, name), rootxpm(NULL), logSize(0),
	toBottom(true), nickPopup(0)
{

	QFont f = KGlobalSettings::fixedFont();
	QStyleSheet *sh = styleSheet();

	style = new QStyleSheetItem(sh, "kt");
	style->setFontFamily(f.family());
	style->setFontSize(f.pointSize());
	style->setFontUnderline(f.underline());
	style->setFontWeight(f.weight());
	style->setFontItalic(f.italic());

	setVScrollBarMode(AlwaysOn);
	setHScrollBarMode(AlwaysOff);
	setReadOnly(true);

	textColorChanged();
	bgColorChanged();
	transparencyChanged();

	clear();
	setFocusPolicy(QWidget::NoFocus);
	setTextFormat(QTextEdit::RichText);
	setWrapPolicy(QTextEdit::AtWordOrDocumentBoundary);

	basic_smile_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|]):-?\\)+|"
			"\\(+-?:(\\s|&nbsp;|$|[`_-\\[\\]|])");
	big_smile_re =  QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|]):-?D(\\s|&nbsp;"
			"|$|[`_-\\[\\]|])");
	confused_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|]):-?(?:\\$|/)"
			"(\\s|&nbsp;|$|[`_-\\[\\]|])");
	sunglasses_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|])B-?\\)"
			"(\\s|&nbsp;|$|[`_-\\[\\]|])");
	cry_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|])(?::'?-?[|(]|T_T)"
			"(\\s|&nbsp;|$|[`_-\\[\\]|])");
	big_eyes_re = QRegExp("(\\s|&nbsp;|^)8-?[|()](\\s|&nbsp;|$|"
			"[`_-\\[\\]|])");
	lol_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|]):-?[pP](\\s|&nbsp;"
			"|$|[`_-\\[\\]|])");
	wink_re = QRegExp("(\\s|&nbsp;|^);-?\\)(\\s|&nbsp;|$|[`_-\\[\\]|])");
	ass_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|])\\(_[o.]_\\)"
			"(\\s|&nbsp;|$|[`_-\\[\\]|])");
	dick_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|])(?:00|8)=+D"
			"(\\s|&nbsp;|$|[`_-\\[\\]|])");
	borne_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|])28(\\s|&nbsp;|$|"
			"[`_-\\[\\]|])");
	turtle_re = QRegExp("(\\s|&nbsp;|^|[`_-\\[\\]|])[tT]+[oO]+[rR]+[tT]"
			"+[uU]+[eE]+!*(\\s|&nbsp;|$|[`_-\\[\\]|])");
	turtle_re.setCaseSensitive(false);

	setNotifyClick(true);
	connect(this, SIGNAL(urlClick ( const QString & )),
		this, SLOT(slotUrl( const QString &  )));
	connect(this, SIGNAL(mailClick ( const QString &, const QString & )),
		this, SLOT(slotMail( const QString &, const QString &  )));

	KwircPreferences *prefs = KwircPreferences::getInstance();

	connect(prefs, SIGNAL(textColorChange()),
			this, SLOT(textColorChanged()));
	connect(prefs, SIGNAL(bgColorChange()), this, SLOT(bgColorChanged()));
	connect(prefs, SIGNAL(transparencyChange()),
			this, SLOT(transparencyChanged()));

	/* Useless
	connect(this, SIGNAL(verticalSliderReleased ()),
			this, SLOT(verticalSliderReleasedSlot ()));
	*/
}

KwircText::~KwircText()
{
}

/* most of this comes from konversation */
QString KwircText::ircEscape(const QString &line)
{
	QString s = QStyleSheet::escape(line);		// esapes all html
	s.replace('\n', "");

//	if(s.find("\x07") != -1 && KwircPreferences::isBeepEnabled())
//		emit beep();

	// replace \003 codes with rich text color codes
	QRegExp colorRegExp("\003(([0-9]|1[0-5]|0[0-9])(,([0-9]|1[0-5])|))?");
	bool openFont = false;
	QString colorString;
	int pos;

	while((pos = colorRegExp.search(s)) != -1) {
		colorString = openFont ? "</font>" : "" ;
		if (!colorRegExp.cap(1).isEmpty()) {
			int foregroundColor = colorRegExp.cap(2).toInt();

			colorString += "<font color=\"#"
				+ QString(colorCodes[foregroundColor]) + "\">";
			openFont = true;
		} else {
			/* if cap(2) is null and openFont == true we are just
			    closing with 003 */
			openFont = false;
		}
		s.replace(pos, colorRegExp.cap(0).length(), colorString);
	}
	// Crude fix
	s.replace('\003', "</font>");
	if(openFont)
		s += "</font>";

	addTags(s, QRegExp("(^|[^a-zA-Z0-9])(\\*\\w+\\*)($|[^a-zA-Z0-9])"),
			"b");
	addTags(s, QRegExp("(^|[^a-zA-Z0-9])(_\\w+_)($|[^a-zA-Z0-9])"), "u");

	replaceDecoration(s,'\x02','b');
	//BUT NOOOOOOOOOOOT, it changes <TAB> to italic
	//replaceDecoration(s,'\x09','i');
	replaceDecoration(s,'\x13','s');
	replaceDecoration(s,'\x15','u');
	replaceDecoration(s,'\x16','b'); // should be reverse
	replaceDecoration(s,'\x1f','u');

	QRegExp pattern("((http://|https://|ftp://|fish://|nntp://"
			"|news://|gopher://|www\\.|ftp\\.)"
			// Username
			"([a-zA-Z0-9_-]+@)?"
			// IP Address
			"([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}|"
			// Decimal IP address
			"[0-9]{1,12}|"
			// Standard host name
			"([\\.@%a-z0-9_-])+\\.[a-z]{2,}"
			// Port number, path to document
			")(:[0-9]{1,5})?(/[^)<>\"'\\s]*)?|"
			// eDonkey2000 links need special treatment
			"ed2k://\\|([^|]+\\|){4})");

	pattern.setCaseSensitive(false);

	pos = 0;
	while(pattern.search(s, pos) != -1) {
		// Remember where we found the url
		pos = pattern.pos();
		// Extract url
		QString url = pattern.capturedTexts()[0];
		QString href(url);

		// clean up href for browser
		if(href.startsWith("www."))
			href = "http://"+href;
		else if(href.startsWith("ftp."))
			href = "ftp://"+href;

		// Fix &amp; back to & in href ... kludgy but I don't know a
		// better way.
		href.replace(QRegExp("&amp;"), "&");
		QString link("<a href=\""+href+"\">"+url+"</a>");

		// replace found url with built link
		s.replace(pos, url.length(), link);
		// next search begins right after the link
		pos += link.length();
	}

	if(KwircSettings::lameMode()) {
		apply_regexp(basic_smile_re, s,
				"<img source=\"basic_smile\">");
		apply_regexp(big_smile_re, s, "<img source=\"big_smile\">");
		apply_regexp(confused_re, s, "<img source=\"confused\">");
		apply_regexp(sunglasses_re, s, "<img source=\"sunglasses\">");
		apply_regexp(cry_re, s, "<img source=\"cry\">");
		apply_regexp(big_eyes_re, s, "<img source=\"bigeyes\">");
		apply_regexp(lol_re, s, "<img source=\"lol\">");
		apply_regexp(wink_re, s, "<img source=\"wink\">");
		// I am an artist
		apply_regexp(ass_re, s, "<img source=\"ass\">");
		apply_regexp(dick_re, s, "<img source=\"dick\">");
		apply_regexp(borne_re, s, "<img source=\"borne\">");
		apply_regexp(turtle_re, s, "<img source=\"turtle\">");
	}

	s.replace("  ", " &nbsp;");
	if (s.at(0) == QChar(' ')) {
		s.replace(0, 1, "&nbsp;");
	}

	return s;
}

void KwircText::apply_regexp(QRegExp &re, QString &s, const QString &str)
{
	int pos = 0;
	while ( pos >= 0 ) {
		pos = re.search(s, pos);
		QString replacement = "";
		if (re.cap(1).compare(" ") == 0)
			replacement += "&nbsp;";
		else
			replacement += re.cap(1);

		replacement += str;

		if (re.cap(2).compare(" ") == 0)
			replacement += "&nbsp;";
		else
			replacement += re.cap(2);
		if ( pos >= 0 ) {
			s.replace(pos, re.cap(0).length(), replacement);
			pos++;      // move along in str
		}
	}
}

QString KwircText::color(const QString &s, const QString &col)
{
	return QString("<font color=\"%1\">%2</font>").arg(col).arg(s);
}

void KwircText::addTags(QString & line, QRegExp regExp, const char * tag)
{
	QString openingTag = QString("<%1>").arg(tag);
	QString closingTag = QString("</%1>").arg(tag);

	int pos = 0;
	while((pos = regExp.search(line, pos)) != -1) {
		QString begin = regExp.cap(1);
		QString body = regExp.cap(2);
		QString text = openingTag + body + closingTag;

		line.replace(pos + begin.length(), body.length(), text);
		pos += begin.length() + text.length();
	}
}

void KwircText::replaceDecoration(QString& line, char decoration,
		char replacement)
{
	int pos;
	bool decorated = false;

	while((pos = line.find(decoration)) != -1) {
		line.replace(pos,1,(decorated) ? QString("</%1>")
				.arg(replacement)
				: QString("<%1>").arg(replacement));
		decorated = !decorated;
	}
	if(decorated)
		line += QString("</%1>").arg(replacement);
}

void KwircText::slotUrl(const QString & link )
{
	if (link.startsWith("nick://")) {
		emit openQuery(link.section('/',2));
	} else
		kapp->invokeBrowser(link);
}

void KwircText::slotMail(const QString &, const QString & link )
{
	kapp->invokeMailer(KURL(link));
}

void KwircText::log(KwircLogType type, const QString &aname, const QString &s1,
		const QString &s2)
{
	QString string, nick, a;
	unsigned int notLevel;

	emit notify(KwircTab::NOT_WALK);

	KwircTheme * theme = KwircPreferences::getCurrentTheme();

	QString arg1 = ircEscape(s1);
	QString arg2 = ircEscape(s2);
	QString name = ircEscape(aname);

	QString prestr = QString();
	bool hl;
	bool you = false;

	switch(type) {
	case LOG_WHOIS:
		string = theme->getComputedString("whois").arg(arg1);
		break;
	case LOG_WHO:
		string = theme->getComputedString("who" ).arg(arg1, arg2);
		break;
	case LOG_RAW:
		string = arg1 + arg2;
		break;
	case LOG_YOUPRIVMSG:
		you = true;
	case LOG_PRIVMSG:
		a = KwircSettings::coloredNicks() ?
		    theme->getColoredNick(arg1): arg1;

		hl = KwircPreferences::isToHighlight(arg2);
		nick = QString("<a href=\"nick://%1\">%2</a>").arg(s1, a);
		if (!hl || you) {
			if (you) {
				string = theme->getComputedString("you_privmsg")
					.arg(nick).arg(arg2);
			} else {
				KNotifyClient::event(0, "PrivMsg", QString("<%1> %2").arg(s1,s2));
				string = theme->getComputedString("privmsg")
					.arg(nick).arg(arg2);
			}
		} else {
			KNotifyClient::event(0, "Hilight", QString("<%1> %2").arg(s1,s2));
			string = theme->getComputedString("privmsg_hl")
				.arg(nick).arg(arg2);
		}
		notLevel = hl ? KwircTab::NOT_HL : KwircTab::NOT_TALK;
		emit notify(notLevel);
		break;
	case LOG_NOTICE:
		hl = KwircPreferences::isToHighlight(arg2);
		if (!hl)
			string = theme->getComputedString("notice").arg(arg1)
				.arg(arg2);
		else
			string = theme->getComputedString("notice_hl")
				.arg(arg1).arg(arg2);
		notLevel = hl ? KwircTab::NOT_HL : KwircTab::NOT_TALK;
		emit notify(notLevel);
		break;
	case LOG_YOUACT:
		you = true;
	case LOG_ACTION:
		hl = KwircPreferences::isToHighlight(arg2);
		if (!hl || you) {
			if (you)
				string = theme->getComputedString("you_act")
					.arg(arg1).arg(arg2);
			else
				string = theme->getComputedString("action")
					.arg(arg1).arg(arg2);
		} else {
			string = theme->getComputedString("action_hl")
				.arg(arg1).arg(arg2);
		}
		notLevel = hl ? KwircTab::NOT_HL : KwircTab::NOT_TALK;
		emit notify(notLevel);
		break;
	case LOG_YOU_JOIN:
		string = theme->getComputedString("you_join").arg(name);
		break;
	case LOG_JOIN:
		string = theme->getComputedString("join").arg(arg1).arg(arg2)
			.arg(name);
		break;
	case LOG_PART:
		string = theme->getComputedString("part").arg(arg1).arg(name)
			.arg(arg2);
		break;
	case LOG_QUIT:
		string = theme->getComputedString("quit").arg(arg1).arg(arg2);
		break;
	case LOG_NICK:
		string = theme->getComputedString("nick").arg(arg1).arg(arg2);
		break;
	case LOG_CTCP:
		string = theme->getComputedString("ctcp").arg(arg1).arg(arg2);
		break;
	case LOG_INVITE:
		string = theme->getComputedString("invite").arg(arg1)
			.arg(arg2);
		break;
	case LOG_KICK:
		string = theme->getComputedString("kick").arg(arg1).arg(arg2);
		break;
	case LOG_BAN:
		string = theme->getComputedString("ban").arg(arg1);
		break;
	case LOG_DEBAN:
		string = theme->getComputedString("deban").arg(arg1);
		break;
	case LOG_OP:
		string = theme->getComputedString("op").arg(arg1);
		break;
	case LOG_DEOP:
		string = theme->getComputedString("deop").arg(arg1);
		break;
	case LOG_VOICE:
		string = theme->getComputedString("voice").arg(arg1);
		break;
	case LOG_DEVOICE:
		string = theme->getComputedString("devoice").arg(arg1)
			.arg(arg2);
		break;
	case LOG_NEWKEY:
		string = theme->getComputedString("newkey").arg(arg1);
		break;
	case LOG_DELKEY:
		string = theme->getComputedString("delkey").arg(arg1);
		break;
	case LOG_TOPIC:
		string = theme->getComputedString("topic").arg(arg1).arg(arg2);
		break;
	case LOG_KILL:
		string = theme->getComputedString("kill").arg(arg1).arg(arg2);
		break;
	case LOG_MODE:
		string = theme->getComputedString("mode").arg(arg1).arg(arg2);
		break;
	case LOG_ERROR:
		string = "<b>*** " + arg1 + "</b>";
		break;
	case LOG_INFO:
		string = "<b>--- " + arg1 + "</b>";
		break;
	default:
		kdError(TEXT_DBG) << "log error" << endl;
		break;
	}

	QTime now = QTime::currentTime();
	QString time = now.toString(theme->getTimestampFormat() );
	string.prepend( QString( "[%1] ").arg( time ) );
        QString disp = QString("<kt>%1</kt>\n").arg(string);

	append(disp);

	if ( (logSize > KwircSettings::maxLogSize() + 1) &&
	     KwircSettings::truncateLog() ){
		doingRemoveParagraph = true;
		removeParagraph(0);
		doingRemoveParagraph = false;
	} else
	     logSize++;

	sync();

	/* Useless
	if (toBottom)
		scrollToBottom();
		*/
	/*	ensureVisible(0, contentsHeight()); */
}

/* Useless
void KwircText::verticalSliderReleasedSlot()
{
	toBottom = (( contentsHeight() - visibleHeight() ) == contentsY());
}
*/

void KwircText::resizeEvent(QResizeEvent *re)
{
	if (toBottom) {
		sync();
		ensureVisible(0, contentsHeight());
	}
	KTextBrowser::resizeEvent(re);
}

void KwircText::ensureCursorVisible()
{
	if (!doingRemoveParagraph)
		QTextEdit::ensureCursorVisible();
}

void KwircText::setNickPopup(NickPopupMenu *popup)
{
	nickPopup = popup;
}

NickPopupMenu *KwircText::getNickPopup()
{
	return nickPopup;
}

QPopupMenu *KwircText::createPopupMenu(const QPoint& pos)
{
	lastLink = anchorAt(pos);
	if(lastLink.isEmpty())
		return KTextBrowser::createPopupMenu(pos);

	if (lastLink.startsWith("nick://")) {
		if (nickPopup) {
			nickPopup->clearNicks();
			nickPopup->addNick(lastLink.section('/',2));
			return nickPopup->duplicate();
		}
	} else {
		// FIX : use also slotMail
		QString txt = lastLink.startsWith("mailto:") ?
			i18n("Open mail composer") : i18n("Open web browser");
        	QPopupMenu *m = new QPopupMenu(this);
        	m->insertItem(txt, this, SLOT(slotUrlPopup()));
	// m->insertItem(tr("Copy location"), this, SLOT(menuCopy())); // FIX
        	return m;
	}
	return KTextBrowser::createPopupMenu(pos);
}

void KwircText::slotUrlPopup()
{
	if(!lastLink.isEmpty())
		emit(slotUrl(lastLink));
}

/*
 * I know, that does not work, but hum, well errrr, it's Qt's fault.
 * We are sexy.
 */
void KwircText::setBackgroundPixmap ( const QPixmap & p)
{
	setPaper(QBrush(QColor(), p));
}

void KwircText::bgColorChanged()
{
	setPaletteBackgroundColor (KwircSettings::bgColor());
}

void KwircText::textColorChanged()
{
	style->setColor(KwircSettings::textColor());
}

void KwircText::transparencyChanged()
{
	if (!KwircSettings::enableTransparency()) {
		if (rootxpm) {
			// FIXME, does not work
			rootxpm->stop();
			setPaper(QBrush(QColor(), QPixmap()));
			repaint();
		}
		return;
	}

	if (!rootxpm) {
		rootxpm = new KRootPixmap(this);
	}

	QColor fadeColor = KwircSettings::fadeColor();
	double fadeStrength = KwircSettings::fadeStrength();
	rootxpm->setFadeEffect(fadeStrength, fadeColor);
	rootxpm->start();
	setStaticBackground(true);
	repaint();
}

#include "kwirctext.moc"
