/*
 * $Id: kwirctext.h,v 1.23 2004/03/09 14:46:31 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar, Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#ifndef KWIRCTEXT_H
#define KWIRCTEXT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <qobject.h>
#include <ktextbrowser.h>
#include <qstring.h>
#include <qobject.h>
#include <qregexp.h>
#include <kwirclog.h>
#include "nickpopupmenu.h"
#include <qpixmap.h>
#include <krootpixmap.h>
#include <qevent.h>

class KwircText : public KTextBrowser
{
	Q_OBJECT
public:
	KwircText(QWidget *parent, const char *name = 0);
	~KwircText();

	/* maybe this has a better place to go to */
	static QString ircEscape(const QString &line);
	void setNickPopup(NickPopupMenu *popup);
	NickPopupMenu *getNickPopup();
	virtual void setBackgroundPixmap ( const QPixmap & p);
private:
	KRootPixmap *rootxpm;
	QStyleSheetItem *style;
	static const char* colorCodes[16];
	static const char* nickColors[11];
	int logSize;
	bool toBottom;
	bool doingRemoveParagraph;
	NickPopupMenu *nickPopup;
	QString lastLink;

	static void apply_regexp(QRegExp &re, QString &, const QString &str);
	static QRegExp basic_smile_re;
	static QRegExp big_smile_re;
	static QRegExp confused_re;
	static QRegExp sunglasses_re;
	static QRegExp cry_re;
	static QRegExp big_eyes_re;
	static QRegExp lol_re;
	static QRegExp wink_re;
	static QRegExp ass_re;
	static QRegExp dick_re;
	static QRegExp borne_re;
	static QRegExp turtle_re;

	QString color(const QString &s, const QString &col);
	static void addTags(QString& line, QRegExp regExp, const char * tag);
	static void replaceDecoration(QString& line, char decoration,
			char replacement);

protected:
	virtual void resizeEvent(QResizeEvent *re);
	QPopupMenu *createPopupMenu(const QPoint& pos);

public slots:
	void log(KwircLogType type, const QString &name, const QString &s1,
			const QString &s2);
	void slotUrl(const QString & link );
	void slotUrlPopup();
	void slotMail(const QString & _null, const QString & link );
	void ensureCursorVisible();

	void textColorChanged();
	void bgColorChanged();
	void transparencyChanged();

private slots:
	/* Useless
	void verticalSliderReleasedSlot(); */
signals:
	void notify(unsigned int);
	void beep();
	void openQuery(const QString &);
};

#endif
