/*
 * $Id: nick.h,v 1.17 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#ifndef NICK_H
#define NICK_H

#include <qptrlist.h>
#include <qobject.h>
#include "kwirclog.h"
#include "destination.h"

class Server;
class Channel;

class Nick : public Destination
{
	Q_OBJECT
	private:
		QString name;
		QString host, ident;
		QPtrList<Channel> channels;
		QString mode;
		QString ircid;
		QString auxinfo;
		QString awayMsg;
		bool away;
	public:
		Nick(Server *s, const QString &);
		~Nick();
		void setInfoString(const QString &n);

		void setAway(bool en);
		void toggleAway(const QString &awaymsg = 0);
		bool isAway();
		void addChannel(Channel *c);
		void removeChannel(Channel *c);
		void recvJoin(Channel *c);
		void recvInvite(Nick *who, Channel *where);
		void recvKill(const QString &what);
		void recvKick(Nick *kicker, Channel *c, const QString &what);
		void recvPart(Channel *c, const QString &what);
		void recvQuit(const QString &what);

		void recvNick(const QString &n);

		void recvPrivmsg(const QString &what);
		void recvAction(const QString &msg);
		void recvCtcp(const QString &ctcp, const QString &what);
		void recvNotice(const QString &what);
		void recvCtcpNotice(const QString &what);

		void isIn(Channel *c);
		void noMoreIn(Channel *c);
		void setHost(const QString &ahost);
		void setIdent(const QString &id);
		const QString &getIrcId();
		const QString &getName();
		const QString &getAuxInfo();
		const QString &getMask();
		bool hasAll();
		bool used();
};

#endif
