/*
 * $Id: nicklistview.cpp,v 1.20 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "nicklistview.h"
#include <qptrlist.h>
#include <qdict.h>
#include "nick.h"
#include <qstringlist.h>
#include <klocale.h>
#include <qmap.h>
#include "kwircpreferences.h"
#include <qpixmap.h>
#include <kiconloader.h>
#include <kdebug.h>
#include "kwircdebug.h"

NicklistView::NicklistView(QWidget *parent, const char *name)
	: KListBox(parent, name)
{
	connect(this, SIGNAL(rightButtonPressed(QListBoxItem *,
					const QPoint &)),
			this, SLOT(popUp(QListBoxItem *, const QPoint &)));
	connect(this, SIGNAL(doubleClicked(QListBoxItem *, const QPoint &)),
			this, SLOT(slotQueryClick(QListBoxItem *,
					const QPoint &)));

	KwircPreferences *prefs = KwircPreferences::getInstance();
	connect(prefs, SIGNAL(textColorChange()),
		this, SLOT(textColorChanged()));
	connect(prefs, SIGNAL(bgColorChange()),
		this, SLOT(bgColorChanged()));

	textColorChanged();
	bgColorChanged();

	setSelectionMode(Extended);
}

NicklistView::~NicklistView()
{
}

void NicklistView::popUp(QListBoxItem *, const QPoint &pos)
{
	nickPopup->clearNicks();
	for (unsigned int i = 0; i < count(); i++) {
		if (isSelected(i))
			nickPopup->addNick(item(i)->text());
	}
	KPopupMenu *menu = nickPopup->duplicate();
	menu->exec(pos);
	delete (KPopupMenu*) menu;
}

void NicklistView::slotQueryClick(QListBoxItem *item, const QPoint &)
{
	QString nick = item->text();
	if (!nick.isEmpty())
		emit triggerQuery(nick);
}

// TODO:recode me

void NicklistView::setChannelUsers(const QPtrList<Nick> &ops,
	const QPtrList<Nick> &voiceds,
	const QPtrList<Nick> &all)
{
	QPixmap opPixmap, voicePixmap, stdPixmap;
	QPixmap opAwayPixmap, voiceAwayPixmap, stdAwayPixmap;
	QPtrList<Nick> opNicks = ops;
	QPtrList<Nick> voicedNicks = voiceds;
	QPtrList<Nick> nicks = all;
	QStringList tmp;
	QMap<QString, QString> caseSort;

	Nick *nick;
	QDict<Nick> str2nick;

	nicksAsString.clear();

	opPixmap = KwircPreferences::getOpPixmap();
	voicePixmap = KwircPreferences::getVoicePixmap();
	stdPixmap = KwircPreferences::getStdPixmap();
	opAwayPixmap = KwircPreferences::getOpAwayPixmap();
	voiceAwayPixmap = KwircPreferences::getVoiceAwayPixmap();
	stdAwayPixmap = KwircPreferences::getStdAwayPixmap();

	// clear the view
	clear();

	// Insert oped nicks
	for (nick = opNicks.first(); nick; nick = opNicks.next()) {

		tmp.append(nick->getStringId().lower());
		caseSort[nick->getStringId().lower()] = nick->getStringId();

		nicksAsString.append(nick->getStringId());
		str2nick.insert(nick->getStringId(), nick);
	}
	tmp.sort();
	for (QStringList::Iterator it = tmp.begin(); it != tmp.end(); ++it) {
		if (str2nick[caseSort[*it]]->isAway()) {
			insertItem(opAwayPixmap, caseSort[*it]);
		} else {
			insertItem(opPixmap, caseSort[*it]);
		}
	}

	// Insert voiced nicks
	tmp.clear();
	for ( nick = voicedNicks.first(); nick; nick = voicedNicks.next() ) {
		if(!nicksAsString.contains(nick->getStringId())) {
			tmp.append(nick->getStringId().lower());
			caseSort[nick->getStringId().lower()]
				= nick->getStringId();
			nicksAsString.append(nick->getStringId());
			str2nick.insert(nick->getStringId(), nick);
		}
	}
	tmp.sort();
	for (QStringList::Iterator it = tmp.begin(); it != tmp.end(); ++it ) {
		if (str2nick[caseSort[*it]]->isAway())
			insertItem(voiceAwayPixmap, caseSort[*it]);
		else
			insertItem(voicePixmap, caseSort[*it]);
	}

	// Insert regular nicks
	tmp.clear();
	for ( nick = nicks.first(); nick; nick = nicks.next() ) {
		if(!nicksAsString.contains(nick->getStringId())) {
			tmp.append(nick->getStringId().lower());
			caseSort[nick->getStringId().lower()]
				= nick->getStringId();
			nicksAsString.append(nick->getStringId());
			str2nick.insert(nick->getStringId(), nick);
		}
	}
	tmp.sort();
	for (QStringList::Iterator it = tmp.begin(); it != tmp.end(); ++it ) {
		if (str2nick[caseSort[*it]]->isAway())
			insertItem(stdAwayPixmap, caseSort[*it]);
		else
			insertItem(stdPixmap, caseSort[*it]);
	}

	emit completionListChange(nicksAsString);
}

const QStringList &NicklistView::getCompletionList()
{
	return nicksAsString;
}

void NicklistView::setNickPopup(NickPopupMenu *popup)
{
	nickPopup = popup;
}

NickPopupMenu *NicklistView::getNickPopup()
{
	return nickPopup;
}

void NicklistView::bgColorChanged()
{
	setPaletteBackgroundColor (KwircSettings::bgColor());
}

void NicklistView::textColorChanged()
{
	setPaletteForegroundColor (KwircSettings::textColor());
}
