/*
 * $Id: nickpopupmenu.cpp,v 1.5 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include <kdebug.h>
#include <klocale.h>
#include "kwircdebug.h"
#include "nickpopupmenu.h"

NickPopupMenu::NickPopupMenu(QWidget *parent, const char *name)
	: KPopupMenu(parent,name)
{
	ctcpMenu = new KPopupMenu(this);
	modeMenu = new KPopupMenu(this);

	modeMenu->insertItem( i18n("&Op"), this, SLOT( slotOp() ) );
	modeMenu->insertItem( i18n("&Deop"), this, SLOT( slotDeop() ) );
	modeMenu->insertSeparator();
	modeMenu->insertItem( i18n("&Voice"), this, SLOT( slotVoice() ) );
	modeMenu->insertItem( i18n("D&evoice"), this, SLOT( slotDevoice() ) );

	ctcpMenu->insertItem( i18n("&Ping"), this, SLOT( slotCtcpPing() ) );
	ctcpMenu->insertItem( i18n("Ve&rsion"), this, SLOT( slotCtcpVersion() ) );

	insertItem( i18n("&CTCP") , ctcpMenu);
	insertSeparator();
	insertItem( i18n("&Modes") , modeMenu);
	insertSeparator();
	insertItem( i18n("&Whois"), this, SLOT( slotWhois() ) );
	insertSeparator();
	insertItem( i18n("&Query"), this, SLOT( slotQuery() ) );
}

KPopupMenu *NickPopupMenu::duplicate()
{
	KPopupMenu *porkymaster = new KPopupMenu();

	porkymaster->insertItem( i18n("&CTCP") , ctcpMenu);
	porkymaster->insertSeparator();
	porkymaster->insertItem( i18n("&Modes") , modeMenu);
	porkymaster->insertSeparator();
	porkymaster->insertItem( i18n("&Whois"), this, SLOT( slotWhois() ) );
	porkymaster->insertSeparator();
	porkymaster->insertItem( i18n("&Query"), this, SLOT( slotQuery() ) );

	return porkymaster;
}

NickPopupMenu::~NickPopupMenu()
{
	if (ctcpMenu)
		delete ctcpMenu;
	if (modeMenu)
		delete modeMenu;
}

void NickPopupMenu::clearNicks()
{
	nicks.clear();
}

void NickPopupMenu::addNick(const QString &text)
{
	nicks.append(text);
}

void NickPopupMenu::slotQuery()
{
	if (nicks.count() == 1)
		emit triggerQuery(*nicks.at(0));
	clearNicks();
}

void NickPopupMenu::slotWhois()
{
	if (nicks.count() == 1)
		emit whois(*nicks.at(0));
	clearNicks();
}

void NickPopupMenu::slotOp()
{
	if (nicks.count() != 0)
		emit op(nicks.join(" "));
	clearNicks();
}

void NickPopupMenu::slotDeop()
{
	if (nicks.count() != 0)
		emit deop(nicks.join(" "));
	clearNicks();
}

void NickPopupMenu::slotVoice()
{
	if (nicks.count() != 0)
		emit voice(nicks.join(" "));
	clearNicks();
}

void NickPopupMenu::slotDevoice()
{
	if (nicks.count() != 0)
		emit devoice(nicks.join(" "));
	clearNicks();
}

void NickPopupMenu::slotCtcpPing()
{
	if (nicks.count() == 1)
		emit ctcp(*nicks.at(0), "PING");
	clearNicks();
}

void NickPopupMenu::slotCtcpVersion()
{
	if (nicks.count() == 1)
		emit ctcp(*nicks.at(0), "VERSION");
	clearNicks();
}

#include "nickpopupmenu.moc"
