/*
 * $Id: rubyscriptmanager.cpp,v 1.28 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "config.h"
#ifdef FEAT_RUBY

#include <qstring.h>
#include <qobject.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <qfile.h>
#include "kwirclog.h"
#include "rubyscriptmanager.h"
#include "command.h"
#include "config.h"
#include "server.h"
#include "nick.h"
#include "channel.h"
#include <qptrlist.h>
#include <kdebug.h>
#include "kwircdebug.h"
#include <kpopupmenu.h>
#include <kaction.h>
#include <klocale.h>
#include <kfiledialog.h>

class RubyCommandItem : public CommandItem
{
	private:
		VALUE lambda;
	public:
		RubyCommandItem(VALUE);
		virtual void execute(ServerElem *se, const QString &);
};

RubyCommandItem::RubyCommandItem(VALUE l)
{
	lambda = l;
}

void RubyCommandItem::execute(ServerElem *se, const QString &cmd)
{
	VALUE rinst = se?se->getRubyInstance():Qnil;
	VALUE array = rb_ary_new();
	rb_ary_push(array, rinst);
	rb_ary_push(array, rb_str_new2(cmd.latin1()));
	RubyScriptManager::apply(lambda, rb_intern("call"), array);
}

VALUE RubyScriptManager::pluginClass = Qnil;
VALUE RubyScriptManager::pluginHash = Qnil;
VALUE RubyScriptManager::destinationArray = Qnil;
VALUE RubyScriptManager::destinationClass = Qnil;
VALUE RubyScriptManager::serverArray = Qnil;
VALUE RubyScriptManager::serverElemClass = Qnil;
VALUE RubyScriptManager::serverClass = Qnil;
VALUE RubyScriptManager::channelArray = Qnil;
VALUE RubyScriptManager::channelClass = Qnil;
VALUE RubyScriptManager::nickArray = Qnil;
VALUE RubyScriptManager::nickClass = Qnil;
QPtrList<KwircRubyPlugin> RubyScriptManager::nativePlugins;
QPtrList<Server> RubyScriptManager::nativeServers;
QPtrList<Channel> RubyScriptManager::nativeChannels;
QPtrList<Nick> RubyScriptManager::nativeNicks;
RubyScriptManager *RubyScriptManager::rubySM = NULL;
bool RubyScriptManager::inited = false;

#define KWIRC_VERSION_MAJOR 0
#define KWIRC_VERSION_MINOR 1
#define KWIRC_VERSION_BUILD 0
#define KWIRC_VERSION_PATCHLEVEL 0
#define KWIRC_VERSION_SHORT "0.1"
#define KWIRC_VERSION_MEDIUM "0.1.0"
#define KWIRC_VERSION_LONG "0.1.0.0-0"
#define KWIRC_VERSION_LONG_DATE "28/02/2003"

extern "C" {
	static VALUE mKWIRC;
	extern VALUE rb_defout;

	static VALUE print_active(VALUE self, VALUE str);
	static VALUE f_p(int argc, VALUE *argv, VALUE self);
}

static VALUE print_active(VALUE, VALUE str)
{
	char *buff, *p;

	str = rb_obj_as_string(str);
	buff = ALLOCA_N(char, RSTRING(str)->len);
	strcpy(buff, RSTRING(str)->ptr);
	p = strchr(buff, '\n');
	if (p)
		*p = '\0';
	RubyScriptManager::printActive(LOG_RAW, buff);
	return Qnil;
}

static VALUE f_p(int argc, VALUE *argv, VALUE)
{
	int i;
	VALUE str = rb_str_new("", 0);

	for (i = 0; i < argc; i++) {
		if (i > 0) rb_str_cat(str, ", ", 2);
		rb_str_concat(str, rb_inspect(argv[i]));
	}
	RubyScriptManager::printActive(LOG_RAW, RSTRING(str)->ptr);
	return Qnil;
}

RubyScriptManager::RubyScriptManager()
	: scriptMenu(NULL)
{
	inited = false;
	init();
}

#define FUNC(a) ((VALUE (*)(...))(a))

void RubyScriptManager::init()
{
	if (inited)
		return;
	ruby_init();
	ruby_init_loadpath();

	mKWIRC = rb_define_module("KWIRC");
	rb_define_const(mKWIRC, "VERSION_MAJOR",
			INT2NUM(KWIRC_VERSION_MAJOR));
	rb_define_const(mKWIRC, "VERSION_MINOR",
			INT2NUM(KWIRC_VERSION_MINOR));
	rb_define_const(mKWIRC, "VERSION_BUILD",
			INT2NUM(KWIRC_VERSION_BUILD));
	rb_define_const(mKWIRC, "VERSION_PATCHLEVEL",
			INT2NUM(KWIRC_VERSION_PATCHLEVEL));
	rb_define_const(mKWIRC, "VERSION_SHORT",
			rb_str_new2(KWIRC_VERSION_SHORT));
	rb_define_const(mKWIRC, "VERSION_MEDIUM",
			rb_str_new2(KWIRC_VERSION_MEDIUM));
	rb_define_const(mKWIRC, "VERSION_LONG",
			rb_str_new2(KWIRC_VERSION_LONG));
	rb_define_const(mKWIRC, "VERSION_LONG_DATE",
			rb_str_new2(KWIRC_VERSION_LONG_DATE));

	rb_defout = rb_obj_alloc(rb_cObject);
	rb_define_singleton_method(rb_defout, "write", FUNC(&print_active), 1);
	rb_define_global_function("p", FUNC(&f_p), -1);

	pluginHash = rb_hash_new();
	rb_gc_register_address(&pluginHash);

	pluginClass = rb_define_class("KwircPlugin", rb_cObject);
	rb_define_method(pluginClass, "register",
			FUNC(&KwircRubyPlugin::registerScript), 1);
	rb_define_method(pluginClass, "command",
			FUNC(&KwircRubyPlugin::registerCommand), 1);

	initServerElemClass();
	initServerClass();
	initDestinationClass();
	initChannelClass();
	initNickClass();
	inited = true;
}

VALUE RubyScriptManager::rubyArray(const QPtrList<Nick> &ary)
{
	QPtrList<Nick> list = ary;
	VALUE rArray = rb_ary_new();

	Nick *n;
	for (n = list.first(); n; n = list.next())
		rb_ary_push(rArray, n->getRubyInstance());
	return rArray;
}

VALUE RubyScriptManager::rubyHash(const QDict<Nick> &hash)
{
	VALUE rHash = rb_hash_new();
	QDictIterator<Nick> it(hash); // See QDictIterator
	for (; it.current(); ++it) {
		rb_hash_aset(rHash, rubyString(it.currentKey()),
				it.current()->getRubyInstance());
	}
	return rHash;
}

VALUE RubyScriptManager::rubyHash(const QDict<Channel> &hash)
{
	VALUE rHash = rb_hash_new();
	QDictIterator<Channel> it(hash); // See QDictIterator
	for (; it.current(); ++it) {
		rb_hash_aset(rHash, rubyString(it.currentKey()),
				it.current()->getRubyInstance());
	}
	return rHash;
}

VALUE RubyScriptManager::rubyString(const QString &str)
{
	if (str.isEmpty())
		return Qnil;
	return rb_str_new2(str.latin1());
}

VALUE RubyScriptManager::safePlugin = Qnil;
VALUE RubyScriptManager::safeMethod = Qnil;
VALUE RubyScriptManager::safeArgArray = Qnil;

VALUE RubyScriptManager::safe_rb_apply()
{
	return rb_apply(safePlugin, safeMethod, safeArgArray);
}

VALUE RubyScriptManager::apply(VALUE obj, VALUE meth, VALUE args)
{
	safePlugin = obj;
	safeMethod = meth;
	safeArgArray = args;

	int state;
	VALUE ret = rb_protect((VALUE (*)(long unsigned int))
			(&RubyScriptManager::safe_rb_apply),
			(VALUE)0, &state);
	if (state) {
		ruby_error(state);
		return Qnil;
	}
	return ret;
}

/*
 * Threads == death
 */
VALUE RubyScriptManager::notify(VALUE method, VALUE args)
{
	VALUE pluginArray = rb_apply(pluginHash, rb_intern("values"),
			rb_ary_new());
	int pluginCount = RARRAY(pluginArray)->len;
	for (int i = 0; i < pluginCount; i++) {
		VALUE plugin = rb_ary_entry(pluginArray, i);
		if (rb_respond_to(plugin, method))
			apply(plugin, method, args);
	}
	return Qnil;
}

const QString RubyScriptManager::notifyYouSay(ServerElem *dest,
		const QString &str)
{
	VALUE ret = Qnil;
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, dest->getRubyInstance());
	rb_ary_push(argArray, rubyString(str));


	VALUE method = rb_intern("saying");
	VALUE pluginArray = rb_apply(pluginHash, rb_intern("values"),
			rb_ary_new());
	int pluginCount = RARRAY(pluginArray)->len;
	for (int i = 0; i < pluginCount; i++) {
		VALUE plugin = rb_ary_entry(pluginArray, i);
		if (rb_respond_to(plugin, method)) {
			VALUE r = apply(plugin, method, argArray);
			if (r != Qnil) {
				rb_ary_store(argArray, 1, r);
				ret = r;
			}
		}
	}

	if (ret == Qnil)
		return str;
	return QString(RSTRING(ret)->ptr);
}

void RubyScriptManager::notifyPrivate(Nick *nick, const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("private"), argArray);
}

void RubyScriptManager::notifyQuit(Nick *nick, const QString &what)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(what));

	notify(rb_intern("quit"), argArray);
}

void RubyScriptManager::notifyPrivateCtcp(Nick *nick, const QString &ctcp,
		const QString &rest)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(ctcp));
	rb_ary_push(argArray, rest.isEmpty() ? Qnil : rubyString(rest));

	notify(rb_intern("private_ctcp"), argArray);
}

void RubyScriptManager::notifyPrivateAction(Nick *nick, const QString &what)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(what));

	notify(rb_intern("private_action"), argArray);
}

void RubyScriptManager::notifyPrivateNotice(Nick *nick, const QString &what)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(what));

	notify(rb_intern("private_notice"), argArray);
}

void RubyScriptManager::notifyPrivateCtcpNotice(Nick *nick,
		const QString &what)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(what));

	notify(rb_intern("private_ctcp_notice"), argArray);
}

void RubyScriptManager::notifyNick(Nick *nick, const QString what)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(what));

	notify(rb_intern("nick"), argArray);
}

void RubyScriptManager::notifyYouJoined(Channel *c)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());

	notify(rb_intern("you_joined"), argArray);
}

void RubyScriptManager::notifyYouLeft(Channel *c, const QString &what)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, rubyString(what));

	notify(rb_intern("you_left"), argArray);
}

void RubyScriptManager::notifyYouWereKicked(Channel *c, Nick *kicker,
		const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, kicker->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("you_were_kicked"), argArray);
}

void RubyScriptManager::notifyPublic(Channel *chan, Nick *nick,
		const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, chan->getRubyInstance());
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("public"), argArray);
}

void RubyScriptManager::notifyPublicNotice(Channel *chan, Nick *nick,
		const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, chan->getRubyInstance());
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("public_notice"), argArray);
}

void RubyScriptManager::notifyPublicCtcp(Channel *c, Nick *nick,
		const QString &ctcp, const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(ctcp));
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("public_ctcp"), argArray);
}

void RubyScriptManager::notifyPublicCtcpNotice(Channel *c, Nick *nick,
		const QString &ctcp, const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(ctcp));
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("public_ctcp_notice"), argArray);
}

void RubyScriptManager::notifyPublicAction(Channel *c, Nick *nick,
		const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("public_action"), argArray);
}

void RubyScriptManager::notifyModeChanged(Channel *c, Nick *nick,
		const QString &mode)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, nick->getRubyInstance());
	rb_ary_push(argArray, rubyString(mode));

	notify(rb_intern("mode_change"), argArray);
}

void RubyScriptManager::notifyJoin(Channel *c, Nick *n)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, n->getRubyInstance());

	notify(rb_intern("join"), argArray);
}

void RubyScriptManager::notifyInvite(Channel *c, Nick *inviter, Nick *invited)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, inviter->getRubyInstance());
	rb_ary_push(argArray, invited->getRubyInstance());

	notify(rb_intern("invite"), argArray);
}

void RubyScriptManager::notifyKill(Channel *c, Nick *killed,
		const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, killed->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("kill"), argArray);
}

void RubyScriptManager::notifyKicked(Channel *c, Nick *kicked, Nick *kicker,
		const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, kicked->getRubyInstance());
	rb_ary_push(argArray, kicker->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("kick"), argArray);
}

void RubyScriptManager::notifyPart(Channel *c, Nick *n, const QString &message)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, n->getRubyInstance());
	rb_ary_push(argArray, rubyString(message));

	notify(rb_intern("part"), argArray);
}

void RubyScriptManager::notifyTopic(Channel *c, const QString &who,
		const QString &top)
{
	VALUE argArray = rb_ary_new();
	rb_ary_push(argArray, c->getRubyInstance());
	rb_ary_push(argArray, rubyString(who));
	rb_ary_push(argArray, rubyString(top));

	notify(rb_intern("topic"), argArray);
}

VALUE RubyScriptManager::serverelem_server(VALUE self)
{
	ServerElem *ptr;
	Data_Get_Struct(self, Server, ptr);
	return ptr->getServer()->getRubyInstance();
}

VALUE RubyScriptManager::server_putserv(VALUE self, VALUE string)
{
	Server *ptr;
	QString str = QString(RSTRING(string)->ptr);
	Data_Get_Struct(self, Server, ptr);
	ptr->putserv(str);
	return Qnil;
}

VALUE RubyScriptManager::server_channels(VALUE self)
{
	Server *ptr;
	Data_Get_Struct(self, Server, ptr);
	const QDict<Channel> &channels = ptr->getChannels();
	return rubyHash(channels);
}

VALUE RubyScriptManager::server_nicks(VALUE self)
{
	Server *ptr;
	Data_Get_Struct(self, Server, ptr);
	const QDict<Nick> &nicks = ptr->getNicks();
	return rubyHash(nicks);
}

VALUE RubyScriptManager::server_channel(VALUE self, VALUE string)
{
	Server *ptr;
	Data_Get_Struct(self, Server, ptr);
	QString str = QString(RSTRING(string)->ptr);
	Channel *c;
	if ((c = ptr->channel(str)) == NULL)
		return Qnil;
	return c->getRubyInstance();
}

VALUE RubyScriptManager::server_nick(VALUE self, VALUE string)
{
	Server *ptr;
	Data_Get_Struct(self, Server, ptr);
	QString str = QString(RSTRING(string)->ptr);
	Nick *n;
	if ((n = ptr->nickFromString(str, false)) == NULL)
		return Qnil;
	return n->getRubyInstance();
}

VALUE RubyScriptManager::server_your_nick(VALUE self)
{
	Server *ptr;
	Data_Get_Struct(self, Server, ptr);
	return ptr->getYourNick()->getRubyInstance();
}

VALUE RubyScriptManager::destination_action(VALUE self, VALUE string)
{
	Destination *ptr;
	QString str = QString(RSTRING(string)->ptr);
	Data_Get_Struct(self, Destination, ptr);
	ptr->sendAction(str);
	return Qnil;
}

VALUE RubyScriptManager::destination_privmsg(VALUE self, VALUE string)
{
	Destination *ptr;
	QString str = QString(RSTRING(string)->ptr);
	Data_Get_Struct(self, Destination, ptr);
	ptr->sendPrivmsg(str);
	return Qnil;
}

VALUE RubyScriptManager::destination_to_s(VALUE self)
{
	Destination *ptr;
	Data_Get_Struct(self, Destination, ptr);
	return rubyString(ptr->getStringId());
}

VALUE RubyScriptManager::channel_nicks(VALUE self)
{
	Channel *ptr;
	Data_Get_Struct(self, Channel, ptr);
	const QPtrList<Nick> &nicks = ptr->getNicks();
	return rubyArray(nicks);
}

VALUE RubyScriptManager::nick_irc_id(VALUE self)
{
	Nick *ptr;
	Data_Get_Struct(self, Nick, ptr);
	return rubyString(ptr->getIrcId());
}

void RubyScriptManager::initServerElemClass()
{
        serverElemClass = rb_define_class("KwircServerElem", rb_cObject);
        rb_define_method(serverElemClass, "server",
			FUNC(&RubyScriptManager::serverelem_server), 0);
}

void RubyScriptManager::initServerClass()
{
	serverArray = rb_ary_new();
	rb_gc_register_address(&serverArray);

        serverClass = rb_define_class("KwircServer", serverElemClass);
        rb_define_method(serverClass, "putserv",
			FUNC(&RubyScriptManager::server_putserv), 1);
        rb_define_method(serverClass, "channels",
			FUNC(&RubyScriptManager::server_channels), 0);
        rb_define_method(serverClass, "channel",
			FUNC(&RubyScriptManager::server_channel), 1);
        rb_define_method(serverClass, "nicks",
			FUNC(&RubyScriptManager::server_nicks), 0);
        rb_define_method(serverClass, "nick",
			FUNC(&RubyScriptManager::server_nick), 1);
        rb_define_method(serverClass, "your_nick",
			FUNC(&RubyScriptManager::server_your_nick), 0);
}

void RubyScriptManager::initDestinationClass()
{
	destinationArray = rb_ary_new();
	rb_gc_register_address(&destinationArray);

        destinationClass = rb_define_class("KwircDestination",
			serverElemClass);
        rb_define_method(destinationClass, "privmsg",
			FUNC(&RubyScriptManager::destination_privmsg), 1);
        rb_define_method(destinationClass, "action",
			FUNC(&RubyScriptManager::destination_action), 1);
        rb_define_method(destinationClass, "to_s",
			FUNC(&RubyScriptManager::destination_to_s), 0);
}

void RubyScriptManager::initChannelClass()
{
	channelArray = rb_ary_new();
	rb_gc_register_address(&channelArray);
        channelClass = rb_define_class("KwircChannel",  destinationClass);
        rb_define_method(channelClass, "nicks",
			FUNC(&RubyScriptManager::channel_nicks), 0);
}

void RubyScriptManager::initNickClass()
{
	nickArray = rb_ary_new();
	rb_gc_register_address(&nickArray);
        nickClass = rb_define_class("KwircNick", destinationClass);
}

VALUE RubyScriptManager::getRubyServerInstance(Server *serv)
{
        VALUE s = Data_Wrap_Struct(serverClass, &RubyScriptManager::mark,
			&RubyScriptManager::free, serv);
	rb_ary_push(serverArray, s);
	nativeServers.append(serv);
	return s;
}

VALUE RubyScriptManager::getRubyChannelInstance(Channel *chan)
{
        VALUE c = Data_Wrap_Struct(channelClass, &RubyScriptManager::mark,
			&RubyScriptManager::free, chan);
	rb_ary_push(channelArray, c);
	nativeChannels.append(chan);
	return c;
}

VALUE RubyScriptManager::getRubyNickInstance(Nick *nick)
{
        VALUE n = Data_Wrap_Struct(nickClass, &RubyScriptManager::mark,
			&RubyScriptManager::free, nick);
	rb_ary_push(nickArray, n);
	nativeNicks.append(nick);
	return n;
}

void RubyScriptManager::mark()
{
}

void RubyScriptManager::free()
{
}

void RubyScriptManager::printActive(KwircLogType type, const QString &str)
{
	emit getInstance()->logActive(type, QString("RubyScript"), str,
			QString::null);
}

VALUE RubyScriptManager::addScript(KwircRubyPlugin *krp, VALUE obj)
{
	KwircRubyPlugin  *p;
	for (p = nativePlugins.first(); p; p = nativePlugins.next()) {
		if (p->getNativeName().compare(krp->getNativeName()) == 0) {
			delScript(p);
			break;
		}
	}
	nativePlugins.append(krp);
	makePopup();

	emit scriptAdded(krp);
	return rb_hash_aset(pluginHash, krp->getRubyName(), obj);
}

VALUE RubyScriptManager::delScript(KwircRubyPlugin *krp)
{
	nativePlugins.remove(krp);
	makePopup();
	emit scriptDeleted(krp);
	return rb_hash_aset(pluginHash, krp->getRubyName(), Qnil);
}

KwircRubyPlugin *RubyScriptManager::getRubyPlugin(VALUE obj)
{
	KwircRubyPlugin *p;
	for (p = nativePlugins.first(); p; p = nativePlugins.next()) {
		if (p->getSelf() == obj) {
			return p;
		}
	}
	return NULL;
}

void RubyScriptManager::makePopup()
{
	scriptMenu->clear();
	scriptMenu->insertItem(i18n("&Load"), this, SLOT(slotLoadScript()));
	scriptMenu->insertSeparator();

	KwircRubyPlugin *p;
	for (p = nativePlugins.first(); p; p = nativePlugins.next()) {
		scriptMenu->insertItem(p->getNativeName(), p->popup());
	}
}

KPopupMenu *RubyScriptManager::popup()
{
	if (scriptMenu)
		return scriptMenu;
	scriptMenu = new KPopupMenu();
	makePopup();
	return scriptMenu;
}

void RubyScriptManager::slotLoadScript()
{
	QStringList qsl = KGlobal::dirs()->findDirs("data", "kwirc/scripts");
	KFileDialog *kfd = new KFileDialog(*qsl.at(0),
			"*.rb", scriptMenu /* grumlbl */,
			"Choose script", true);
	connect(kfd, SIGNAL(okClicked()),
			this, SLOT(loadSelectedFile()));
	kfd->show();
}

void RubyScriptManager::loadSelectedFile()
{
	KFileDialog *kdf = (KFileDialog*) sender();
	QString name = kdf->selectedFile();
	loadScript(name);
}

void RubyScriptManager::ruby_error(int state)
{
	extern VALUE ruby_errinfo;
	VALUE eclass;
	VALUE einfo;
	char buff[BUFSIZ];

#define TAG_RETURN	0x1
#define TAG_BREAK	0x2
#define TAG_NEXT	0x3
#define TAG_RETRY	0x4
#define TAG_REDO	0x5
#define TAG_RAISE	0x6
#define TAG_THROW	0x7
#define TAG_FATAL	0x8
#define TAG_MASK	0xf

	switch (state) {
		case TAG_RETURN:
			printActive(LOG_RAW, QString("unexpected return"));
			break;
		case TAG_NEXT:
			printActive(LOG_RAW, "unexpected next");
			break;
		case TAG_BREAK:
			printActive(LOG_RAW, "unexpected break");
			break;
		case TAG_REDO:
			printActive(LOG_RAW, "unexpected redo");
			break;
		case TAG_RETRY:
			printActive(LOG_RAW, "retry outside of rescue clause");
			break;
		case TAG_RAISE:
		case TAG_FATAL:
			eclass = CLASS_OF(ruby_errinfo);
			einfo = rb_obj_as_string(ruby_errinfo);
			if (eclass == rb_eRuntimeError
					&& RSTRING(einfo)->len == 0) {
				printActive(LOG_RAW, "unhandled exception");
			}
			else {
				VALUE epath;
				char *p;

				epath = rb_class_path(eclass);
				snprintf(buff, BUFSIZ, "%s: %s",
						RSTRING(epath)->ptr,
						RSTRING(einfo)->ptr);
				p = strchr(buff, '\n');
				if (p) *p = '\0';
				printActive(LOG_RAW,buff);
			}
			break;
		default:
			snprintf(buff, BUFSIZ, "unknown longjmp status %d",
					state);
			printActive(LOG_RAW,buff);
			break;
	}
}

bool RubyScriptManager::loadScript(const QString &path)
{
	QString fullPath;
	if (QFile::exists(path)) {
		fullPath = path;
	} else {
		fullPath = KGlobal::dirs()->findResource("data",
			QString("kwirc/scripts/") + path);
	}

	int state;
	rb_load_protect(rubyString(fullPath), 0, &state);
	if (state) {
		ruby_error(state);
		kdError(RUBY_DBG) << fullPath << " can't be loaded" << endl;
		return false;
	}
	return true;
}

RubyScriptManager *RubyScriptManager::getInstance()
{
	if (rubySM)
		return rubySM;
	return rubySM = new RubyScriptManager();
}

/*****************************************************************************/

KwircRubyPlugin::KwircRubyPlugin(VALUE aself, VALUE aname)
{
	self = aself;
	name = aname;
	qname = QString(RSTRING(aname)->ptr);

	pop = new KPopupMenu();

	actionUnload = new KAction(i18n("&Unload %1").arg(qname), 0,
				this, SLOT(unload()), pop,
				QString("Unload %1").arg(qname).latin1());
	actionUnload->plug(pop);
}

VALUE KwircRubyPlugin::realRegisterCommand(VALUE, VALUE name)
{
	RubyCommandItem *rci = new RubyCommandItem(rb_f_lambda());
	QString cmd = QString(RSTRING(name)->ptr);
	bool ret = Command::registerCommand(cmd, rci);
	if (ret) {
		registeredCommands.append(cmd);
		return Qtrue;
	}
	return Qfalse;
}

VALUE KwircRubyPlugin::registerCommand(VALUE self, VALUE name)
{
	KwircRubyPlugin *rp = RubyScriptManager::getRubyPlugin(self);
	return rp->realRegisterCommand(self, name);
}

VALUE KwircRubyPlugin::registerScript(VALUE self, VALUE name)
{
	KwircRubyPlugin *krp = new KwircRubyPlugin(self, name);
	RubyScriptManager::getInstance()->addScript(krp, self);
	return Qnil;
}

VALUE KwircRubyPlugin::getRubyName()
{
	return name;
}

const QString &KwircRubyPlugin::getNativeName()
{
	return qname;
}

/*
 * and leak
 * FIXME: the commmand unregistering does not seem to work, leading to a crash
 */
void KwircRubyPlugin::unload()
{
	for (QStringList::Iterator it = registeredCommands.begin();
			it != registeredCommands.end(); ++it )
		Command::unregisterCommand(*it);

	RubyScriptManager::getInstance()->delScript(this);
}

KPopupMenu *KwircRubyPlugin::popup()
{
	return pop;
}

VALUE KwircRubyPlugin::getSelf()
{
	return self;
}


#include "rubyscriptmanager.moc"
#endif // FEAT_RUBY
