/*
 * $Id: rubyscriptmanager.h,v 1.14 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "config.h"
#ifdef FEAT_RUBY
#ifndef RUBYSCRIPTMANAGER_H
#define RUBYSCRIPTMANAGER_H
#include <ruby.h>
#include <qobject.h>
#include <kwirclog.h>
#include <qptrlist.h>

class ServerElem;
class Server;
class Nick;
class Channel;
class KPopupMenu;
class KAction;

class KwircRubyPlugin : public QObject
{
	Q_OBJECT
	private:
		QString qname;
		VALUE name;
		VALUE self;
		QStringList registeredCommands;
		KPopupMenu *pop;
		KAction *actionUnload;
	public:
		KwircRubyPlugin(VALUE self, VALUE name);
		VALUE getRubyName();
		VALUE getSelf();
		const QString &getNativeName();
		static VALUE registerScript(VALUE self, VALUE name);
		static VALUE registerCommand(VALUE self, VALUE name);
		VALUE realRegisterCommand(VALUE, VALUE name);
		KPopupMenu *popup();
	public slots:
		void unload();
};

class RubyScriptManager : public QObject
{
	Q_OBJECT
	private:
		static VALUE pluginHash;
		static VALUE pluginClass;
		static QPtrList <KwircRubyPlugin> nativePlugins;
		static VALUE serverArray;
		static VALUE serverElemClass;
		static VALUE serverClass;
		static VALUE destinationArray;
		static VALUE destinationClass;
		static VALUE channelArray;
		static VALUE channelClass;
		static VALUE nickArray;
		static VALUE nickClass;


		static VALUE safePlugin;
		static VALUE safeArgArray;
		static VALUE safeMethod;
		static QPtrList<Server> nativeServers;
		static QPtrList<Nick> nativeNicks;
		static QPtrList<Channel> nativeChannels;
		static RubyScriptManager *rubySM;
		static bool inited;
		static void initServerElemClass();
		static void initServerClass();
		static void initDestinationClass();
		static void initChannelClass();
		static void initNickClass();
		static VALUE serverelem_server(VALUE self);
		static VALUE server_putserv(VALUE self, VALUE string);
		static VALUE server_channels(VALUE self);
		static VALUE server_channel(VALUE self, VALUE string);
		static VALUE server_nicks(VALUE self);
		static VALUE server_nick(VALUE self, VALUE string);
		static VALUE server_your_nick(VALUE self);
		static VALUE channel_nicks(VALUE self);
		static VALUE destination_privmsg(VALUE self, VALUE string);
		static VALUE destination_action(VALUE self, VALUE string);
		static VALUE destination_to_s(VALUE self);
		static VALUE nick_irc_id(VALUE self);
		static void mark();
		static void free();
		KPopupMenu *scriptMenu;
		RubyScriptManager();
		void makePopup();
	public:
		VALUE addScript(KwircRubyPlugin *krp, VALUE obj);
		VALUE delScript(KwircRubyPlugin *krp);

		static RubyScriptManager *getInstance();
		static void ruby_error(int);
		static void printActive(KwircLogType, const QString &);
		static void init();
		static VALUE getRubyServerInstance(Server *serv);
		static VALUE getRubyNickInstance(Nick *nick);
		static VALUE getRubyChannelInstance(Channel *chan);

		static VALUE rubyString(const QString &str);
		static VALUE rubyHash(const QDict<Channel> &hash);
		static VALUE rubyHash(const QDict<Nick> &hash);
		static VALUE rubyArray(const QPtrList<Nick> &ary);
		static bool loadScript(const QString &what);

		static VALUE notify(VALUE method, VALUE args);

		static void notifyPrivate(Nick *nick, const QString &message);
		static void notifyQuit(Nick *nick, const QString &what);
		static void notifyPrivateCtcp(Nick *nick,
				const QString &ctcp, const QString &rest);
		static void notifyPrivateAction(Nick *nick,
				const QString &what);
		static void notifyPrivateNotice(Nick *nick,
				const QString &what);
		static void notifyPrivateCtcpNotice(Nick *nick,
				const QString &what);
		static void notifyNick(Nick *nick, const QString what);
		static void notifyYouJoined(Channel *c);
		static void notifyYouLeft(Channel *c, const QString &what);
		static void notifyYouWereKicked(Channel *c, Nick *kicker,
				const QString &message);
		static void notifyPublic(Channel *chan, Nick *nick,
				const QString &message);
		static void notifyPublicNotice(Channel *chan, Nick *nick,
				const QString &ctcp);
		static void notifyPublicCtcp(Channel *c, Nick *nick,
				const QString &ctcp, const QString &message);
		static void notifyPublicCtcpNotice(Channel *c, Nick *nick,
				const QString &ctcp, const QString &message);
		static void notifyPublicAction(Channel *c, Nick *nick,
				const QString &message);
		static void notifyModeChanged(Channel *c, Nick *nick,
				const QString &mode);
		static void notifyJoin(Channel *c, Nick *n);
		static void notifyInvite(Channel *c, Nick *inviter,
				Nick *invited);
		static void notifyKill(Channel *c, Nick *killed,
				const QString &message);
		static void notifyKicked(Channel *c, Nick *kicked,
				Nick *kicker, const QString &message);
		static void notifyPart(Channel *c, Nick *n,
				const QString &message);
		static void notifyTopic(Channel *c, const QString &who,
				const QString &top);

		static const QString notifyYouSay(ServerElem*,
				const QString &str);

		static VALUE safe_rb_apply();
		static VALUE apply(VALUE obj, VALUE meth, VALUE args);

		KPopupMenu *popup();
		static KwircRubyPlugin *getRubyPlugin(VALUE obj);
	signals:
		void logActive(KwircLogType, const QString &,
				const QString &, const QString &);
		void scriptAdded(KwircRubyPlugin *);
		void scriptDeleted(KwircRubyPlugin *);
	private slots:
		void slotLoadScript();
		void loadSelectedFile();
};
#endif
#endif // FEAT_RUBY
