/*
 * $Id: serverelem.h,v 1.10 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#ifndef SERVERELEM_H
#define SERVERELEM_H

#include <qobject.h>
#include <qstring.h>
#include <kwirclog.h>
#include "config.h"
#ifdef FEAT_RUBY
#include <ruby.h>
#endif

class Server;

class ServerElem : public QObject
{
	Q_OBJECT
public:
	enum elem_type {
		TYPE_CHANNEL,
		TYPE_NICK,
		TYPE_SERVER
	};
private:
	elem_type type;
	QString stringId;
#ifdef FEAT_RUBY
	VALUE rubyInstance;
#endif
protected:
	void setStringId(const QString &);
#ifdef FEAT_RUBY
	void setRubyInstance(VALUE);
#endif
public:
#ifdef FEAT_RUBY
	VALUE getRubyInstance();
#endif
	ServerElem(const QString &name, elem_type type);
	~ServerElem();

	const QString &getStringId();
	elem_type getType();
	virtual Server *getServer() = 0;
	bool isDestination();
	void clear();
signals:
	void stringIdChange(const QString &);
	void log(KwircLogType, const QString &, const QString &,
			const QString &);
	void logActive(KwircLogType, const QString &, const QString &,
			const QString &);
	void clearLog();
public slots:
	virtual void input(const QString &input);
	QStringList complete(const QString &str);
};

#endif
