/*
 * $Id: servermanager.ui.h,v 1.12 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

#include <qcheckbox.h>
#include <config.h>

void ServerManager::init()
{
    settingsList.setAutoDelete(true);
    wdgList.setAutoDelete(true);
    managerList.setAutoDelete(true);

    maxServerNumber = 0;

    QStringList serverGroups = KwircSettings::connectOrder();

    for (QStringList::Iterator it = serverGroups.begin(); it !=serverGroups .end(); ++it ) {
	QRegExp re("Server (\\d+)");
	re.search(*it);
	int serverNumber = re.cap(1).toInt();
	if (serverNumber > maxServerNumber)
	    maxServerNumber = serverNumber;

	addServerTab(*it, false);
    }
}

void ServerManager::addNewServerTab()
{
    maxServerNumber++;
    QString group = QString("Server %1").arg(maxServerNumber);
    addServerTab(group, true);
}

void ServerManager::addServerTab( const QString &group , bool show)
{
    ServerSettings *settings = new ServerSettings(group);
    settings->readConfig();
    ServerSettingsWdg *wdg = new ServerSettingsWdg();
    KConfigDialogManager *manager = new KConfigDialogManager(wdg, settings);
    connect(manager, SIGNAL(widgetModified ()), this, SLOT(widgetChangedSlot()));
    manager->updateWidgets();

    managerList.append(manager);
    wdgList.append(wdg);
    settingsList.append(settings);

#ifndef HAVE_SSL
	wdg->kcfg_useSSL->setEnabled(false);
	wdg->kcfg_useSSL->setText(i18n("SSL support was disabled during compilation"));
#endif

    QObject * qobj = (QObject *) wdg->kcfg_label;
    connect(qobj, SIGNAL(textChanged (const QString &)),
	    this, SLOT(changeCurrentTabLabel(const QString &)));
    QIconSet iconSet= SmallIconSet("server");
    serverTabs->addTab(wdg, iconSet, settings->label());

    if (show) {
	serverTabs->showPage((QWidget *)wdg);
    }
}

void ServerManager::removeCurrentServerTab()
{
    QWidget * current = serverTabs->currentPage();

    int pos = wdgList.find(current);
    serverTabs->removePage(current);

    managerList.remove(pos);
    settingsList.remove(pos);
    wdgList.remove(pos);
}

void ServerManager::handleConnectionRequest()
{
    QWidget * current = serverTabs->currentPage();
    int pos = wdgList.find(current);
    ServerSettings *settings = settingsList.at(pos);

    if (settings) {
	emit connectionRequest(settings);
    }
}

void ServerManager::changeCurrentTabLabel( const QString &label )
{
	serverTabs->setTabLabel(serverTabs->currentPage(), label);
}

void ServerManager::moveCurrentTabRight()
{
	int pos = serverTabs->currentPageIndex();
	if (pos == serverTabs->count() - 1) {
		return;
	}
	pos++;
	QWidget *currentPage = serverTabs->currentPage();
	QString label = serverTabs->tabLabel(currentPage);
	QIconSet iconSet = serverTabs->tabIconSet(currentPage);
	serverTabs->removePage(currentPage);
	serverTabs->insertTab(currentPage, iconSet, label, pos);
	serverTabs->setCurrentPage(pos);
	saveTabOrder();
}

void ServerManager::moveCurrentTabLeft()
{
	int pos = serverTabs->currentPageIndex();
	if (pos == 0) {
		return;
	}
	pos--;
	QWidget *currentPage = serverTabs->currentPage();
	QString label = serverTabs->tabLabel(currentPage);
	QIconSet iconSet = serverTabs->tabIconSet(currentPage);
	serverTabs->removePage(currentPage);
	serverTabs->insertTab(currentPage, iconSet, label, pos);
	serverTabs->setCurrentPage(pos);
	saveTabOrder();
}

void ServerManager::saveTabOrder()
{
    QStringList groupList;

    int count = serverTabs->count();
    for (int i=0; i<count; i++) {
	QWidget * widget = serverTabs->page(i);
	int pos = wdgList.find(widget);
	ServerSettings *settings = settingsList.at(pos);

	if (settings) {
	    groupList.append(settings->currentGroup());
	}
    }

    KwircSettings::setConnectOrder(groupList);
}

void ServerManager::saveSettings()
{
    saveTabOrder();
    ServerSettings *settings;
    KConfigDialogManager *manager;
    for (manager = managerList.first(); manager; manager = managerList.next()) {
	manager->updateSettings();
    }

    for (settings = settingsList.first(); settings; settings = settingsList.next()) {
	settings->writeConfig();
    }
}


void ServerManager::widgetChangedSlot()
{
    // FIXME
    bool hasChanged = false;
    KConfigDialogManager *manager;
    for (manager = managerList.first(); manager && !hasChanged; manager = managerList.next()) {
	if (manager->hasChanged())
	    hasChanged = true;
    }
    emit changed(hasChanged);
}
