/*
 * $Id: servertab.cpp,v 1.18 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "servertab.h"
#include <kwircpreferences.h>
#include "server.h"
#include "channeltab.h"
#include "querytab.h"

ServerTab::ServerTab(Server *s, const QString& text, const KShortcut& cut)
	 :KwircTab(s, text, KwircPreferences::getNotIcon(NOT_RESET_SERV), cut)
{
	setResetIcon();
}

ServerTab::~ServerTab()
{
}

void ServerTab::addQueryTab(QueryTab *qt)
{
	queries.append(qt);
}

void ServerTab::addChannelTab(ChannelTab *ct)
{
	channels.append(ct);
}

ChannelTab *ServerTab::channelTab(const QString &channame)
{
	ChannelTab *it;
	for (it = channels.first(); it; it = channels.next()) {
		if ( QString( it->name() ).lower() == channame.lower()) {
		//if (it->text().lower() == channame.lower()) {
			return it;
		}
	}
	return NULL;
}

QueryTab *ServerTab::queryTab(const QString &name)
{
	QueryTab *it;
	for (it = queries.first(); it; it = queries.next()) {
		if ( QString( it->name() ).lower() == name.lower()) {
		//if (it->text().lower() == name.lower()) {
			return it;
		}
	}
	return NULL;
}

void ServerTab::removeQueryTab(QueryTab *qt)
{
	queries.remove(qt);
}

void ServerTab::removeChannelTab(ChannelTab *ct)
{
	channels.remove(ct);
}

const QPtrList<ChannelTab> &ServerTab::getChannelTabs()
{
	return channels;
}

const QPtrList<QueryTab> &ServerTab::getQueryTabs()
{
	return queries;
}

void ServerTab::setResetIcon()
{
	currentLevel = NOT_RESET_SERV;
	setIcon(KwircPreferences::getNotIcon(NOT_RESET_SERV));
}

void ServerTab::close()
{
	channels.clear();
	queries.clear();

	KwircTab::close();
}

KwircTab *ServerTab::lastTab()
{
	if (queries.last())
		return queries.last();
	if (channels.last())
		return channels.last();
	return this;
}

KwircTab *ServerTab::precTab(KwircTab *tab)
{
	KwircTab *it;
	KwircTab *ret = this;
	for (it = channels.first(); it; it = channels.next()) {
		if (it == tab)
			return ret;
		ret = it;
	}
	for (it = queries.first(); it; it = queries.next()) {
		if (it == tab)
			return ret;
		ret = it;
	}
	return NULL;
}

KwircTab *ServerTab::nextTab(KwircTab *tab)
{
	KwircTab *it;
	KwircTab *ret = this;
	if (tab == this) {
		if (channels.first() != NULL)
			return channels.first();
		return queries.first();
	}
	for (it = channels.first(); it; it = channels.next()) {
		ret = it;
		if (it == tab) {
			it = channels.next();
			if (!it)
				return queries.first();
			return it;
		}
	}
	for (it = queries.first(); it; it = queries.next()) {
		ret = it;
		if (it == tab)
			return queries.next();
	}
	return NULL;
}

ServerTab *ServerTab::getServerTab()
{
	return this;
}
