/*
 * $Id: themeeditor.ui.h,v 1.14 2004/03/07 15:16:07 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 Julien Perrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

const char * ThemeEditor::keyToLabel[NB_STRINGS][2] = {
	{"privmsg"    , "Private message"},
	{"privmsg_hl" , "Private message (hl)"},
	{"notice"     , "Notice"},
	{"notice_hl"  , "Notice (hl)"},
	{"action"     , "Action"},
	{"action_hl"  , "Action (hl)"},
	{"you_join"   , "You join"},
	{"join"       , "Join"},
	{"part"       , "Part"},
	{"quit"       , "Quit"},
	{"nick"       , "Nick"},
	{"ctcp"       , "Ctcp"},
	{"invite"     , "Invite"},
	{"kick"       , "Kick"},
	{"ban"        , "Ban"},
	{"deban"      , "Deban"},
	{"op"         , "Op"},
	{"deop"       , "De-op"},
	{"voice"      , "Voice"},
	{"devoice"    , "Un-voice"},
	{"newkey"     , "New key"},
	{"delkey"     , "Del key"},
	{"topic"      , "Topic"},
	{"kill"       , "Kill"},
	{"mode"       , "Mode"},
	{"whois"      , "Whois"},
	{"who"        , "Who"},
	{"you_privmsg", "You say"},
	{"you_act"    , "You act"}
};

struct preview_args {
	uint nb;
	QString args[3];
};
const struct preview_args ThemeEditor::previewArgs[NB_STRINGS] = {
	{2, {"nohar", "all your base belong to us", ""}}, //privmsg
	{2, {"nohar", "all your base belong to us", ""}}, //privmsg_hl
	{2, {"a", "b", ""}}, //notice
	{2, {"a", "b", ""}}, //notice_hl
	{2, {"nohar", "is so sexy", ""}}, //action
	{2, {"nohar", "is so sexy", ""}}, //action_hl
	{1, {"#sexy-boys", "", ""}}, //you_join
	{3, {"nohar", "nohar@i.am.so.l33t.com", "#sexy-boys"}}, //join
	{3, {"nohar", "#vodka", "Got root ?"}}, //part
	{2, {"nohar", "Got root ?", ""}}, //quit
	{2, {"nohar", "nohair", ""}}, //nick
	{2, {"a", "b", ""}}, //ctcp
	{2, {"a", "b", ""}}, //invite
	{2, {"Atchoum", "IDLE!", ""}}, //kick
	{1, {"a", "", ""}}, //ban
	{1, {"a", "", ""}}, //deban
	{1, {"a", "", ""}}, //op
	{1, {"a", "", ""}}, //deop
	{1, {"a", "", ""}}, //voice
	{1, {"a", "", ""}}, //devoice
	{1, {"a"}}, //newkey
	{0, {"", "", ""}}, //delkey
	{2, {"a", "b", ""}}, //topic
	{2, {"a", "b", ""}}, //kill
	{2, {"a", "b", ""}},
	{1, {"a"}}, //whois
	{2, {"a", "b"}}, //who
	{2, {"a", "b"}}, //you_privmsg
	{2, {"a", "b"}} //you_act
};

void ThemeEditor::init()
{
	enabled = false;

	QString themeName = KwircSettings::themeName();

	textPreview->setTextFormat(Qt::RichText);
	saveButton->setEnabled(false);

	loadTheme(themeName);
}

void ThemeEditor::loadTheme( QString path )
{
	QString fullPath;
	if (QFile::exists(path)) {
		fullPath = path;
	} else {
		KStandardDirs *dirs = KGlobal::dirs();
		fullPath = dirs->findResource("data",
					      "kwirc/themes/" + path);
		if (fullPath == QString::null) {
			kdError() << "File not found ! : " << path << endl;
			themeName->setText("No theme loaded");
			return;
		}
	}

	themeName->setText(fullPath);
	pathName = fullPath;
	config = new KConfig(pathName);
	currentTheme = new KwircTheme(config);

	// Loading the color values from the theme file and setting them
	// to the widgets
	QColor defaultColor = Qt::black;
	QObject *obj;
	KColorButton *colorButton;
	QObjectList *colorButtonList = queryList("KColorButton", 0, TRUE);
	QObjectListIt it(*colorButtonList);

	config->setGroup("Theme");
	timestampFormat->setText(config->readEntry("timestampFormat","hh:mm"));
	while ( (obj = it.current()) != 0 ) {
		++it;
		colorButton = (KColorButton *) obj;
		QColor color = config->readColorEntry(colorButton->name(), &defaultColor);
		colorButton->setColor(color);
		connect(colorButton, SIGNAL(changed(const QColor &)),
			SLOT(handleColorChange(const QColor &)));
		connect(colorButton, SIGNAL(changed(const QColor &)),
			SLOT(changeSaveButtonState()));
	}
	delete colorButtonList;

	for (int i=0; i<NB_STRINGS; i++) {
		QString key = QString(keyToLabel[i][0]);
		QString label = QString(keyToLabel[i][1]);
		formatStrings[key] = currentTheme->getFormatString(key);
		formatStringListBox->insertItem(label);
	}

	connect(formatStringEditor, SIGNAL(textChanged ( const QString & )),
		SLOT(changeSaveButtonState()));

	enabled = true;
}

void ThemeEditor::saveTheme(KConfig *destConfig)
{
	destConfig->setGroup("Theme");

	destConfig->writeEntry("timestampFormat", timestampFormat->text());

	QObject *obj;
	KColorButton *colorButton;
	QObjectList *colorButtonList = queryList("KColorButton", 0, TRUE);
	QObjectListIt it(*colorButtonList);

	while ( (obj = it.current()) != 0 ) {
		++it;
		colorButton = (KColorButton *) obj;
		destConfig->writeEntry(colorButton->name(), colorButton->color());
	}
	delete colorButtonList;

	QMap<QString,QString>::Iterator it2;
	for (it2 = formatStrings.begin(); it2 != formatStrings.end(); it2++) {
		destConfig->writeEntry(it2.key(), it2.data());
	}

	destConfig->sync();
	saveButton->setEnabled(false);
}

void ThemeEditor::saveTheme( QString path )
{
	if (!enabled)
	    return;

	KConfig * destConfig = new KConfig(path);
	saveTheme(destConfig);
}

void ThemeEditor::saveTheme()
{
	if (!enabled)
		return;

	saveTheme(config);
}

void ThemeEditor::changeSaveButtonState()
{
	if (!enabled)
		return;

	saveButton->setEnabled(TRUE);
}

void ThemeEditor::handleListBoxSelection(int index)
{
	if (!enabled)
		return;

	if (index < NB_STRINGS) {
		QString key = keyToLabel[index][0];
		formatStringEditor->setText(formatStrings[key]);
	}
}

void ThemeEditor::handleFormatStringChange()
{
	if (!enabled)
		return;
	int i = formatStringListBox->currentItem();
	if (i < NB_STRINGS) {
		QString key = keyToLabel[i][0];
		QString newValue = formatStringEditor->text();
		formatStrings[key] = newValue;
		currentTheme->setFormatString(key, newValue);
		updatePreview();
	}
}

void ThemeEditor::handleSaveAsRequest()
{
	QString path = KFileDialog::getSaveFileName();
	if (!path.isEmpty()) {
		saveTheme(path);
	}
}

void ThemeEditor::handleLoadThemeRequest()
{
	QString path = KFileDialog::getOpenFileName();
	if (!path.isEmpty()) {
		if (enabled) {
			delete config;
			delete currentTheme;
			formatStringListBox->clear();
		}
		loadTheme(path);
	}
}

void ThemeEditor::handleReloadRequest()
{
	if (enabled) {
		delete config;
		delete currentTheme;
		formatStringListBox->clear();
	}
	loadTheme(pathName);
}

void ThemeEditor::handleColorChange( const QColor &newColor )
{
	if (enabled) {
		const QObject *button = sender();
		if (!button)
			return;
		QString name = button->name();
		QRegExp re("color(\\d+)");
		int pos = re.search(name);
		if (pos != -1) {
			QString number = re.cap(1);
			currentTheme->setColor(number, newColor);
			updatePreview();
		}
	}
}

void ThemeEditor::updatePreview()
{
	int i = formatStringListBox->currentItem();
	if (i != -1) {
		QString key = keyToLabel[i][0];
		const struct preview_args arg = previewArgs[i];
		QString preview = currentTheme->getComputedString(key);
		for (uint j = 0; j<arg.nb; j++) {
			preview = preview.arg(arg.args[j]);
		}

		textPreview->setText(preview);
	}
}
