/*
 * $Id: topicview.cpp,v 1.16 2004/03/09 14:47:27 blackmor Exp $
 *
 * This file is part of the kwirc project
 * Copyright (C) 2003 nohar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */
#include "topicview.h"
#include "kwirctext.h"
#include <qlayout.h>
#include <qevent.h>
#include <kdebug.h>
#include <klocale.h>
#include "kwircdebug.h"
#include "kwircpreferences.h"

TopicView::TopicView(QWidget *parent, char *name)
	:KTextBrowser(parent, name)
{
	setVScrollBarMode(QScrollView::AlwaysOff);
	setHScrollBarMode(QScrollView::AlwaysOff);

	setTextFormat(QTextEdit::RichText);
	setReadOnly(true);
	setTopic(i18n("No topic set"));

	KwircPreferences *prefs = KwircPreferences::getInstance();
	connect(prefs, SIGNAL(textColorChange()),
		this, SLOT(textColorChanged()));
	connect(prefs, SIGNAL(bgColorChange()),
		this, SLOT(bgColorChanged()));

	textColorChanged();
	bgColorChanged();
}

TopicView::~TopicView()
{
}

void TopicView::autoSize()
{
	setMaximumHeight(heightForWidth(width()) + 5); // FIXME
}

void TopicView::resizeEvent(QResizeEvent *re)
{
	QTextEdit::resizeEvent(re);
	autoSize();
}

void TopicView::setTopic(const QString &atopic)
{
	topic = atopic;
	setText(KwircText::ircEscape(atopic));
	sync();
	autoSize();
}

void TopicView::setEditableMode(bool enable)
{
	setReadOnly(!enable);
	if (enable) {
		setTextFormat(QTextEdit::PlainText);
		setText(topic);
	} else {
		setTextFormat(QTextEdit::RichText);
		setText(KwircText::ircEscape(topic));
	}
	sync();
	autoSize();
}

/* does not work:
void TopicView::mousePressEvent(QMouseEvent *e)
{
	setEditableMode(true);
	KTextEdit::mousePressEvent(e);
} */

bool TopicView::eventFilter(QObject *obj, QEvent *e)
{
	if (!isReadOnly()) {
		if (e->type() == QEvent::KeyPress) {
			QKeyEvent *keye = (QKeyEvent*) e;
			if (keye->key() == Key_Return) {
				//grabWidget->releaseMouse();
				if (QString::compare(text(), topic) != 0) {
					topic = text();
					emit topicChanged(topic);
				}
				setEditableMode(false);
				return true;
			} else if (keye->key() == Key_Escape) {
				//grabWidget->releaseMouse();
				setEditableMode(false);
				return true;
			}
		} else if (e->type() == QEvent::MouseButtonRelease) {
			QMouseEvent *me = (QMouseEvent*) e;
			if (!rect().contains(me->pos())) {

				//grabWidget->releaseMouse();
				setEditableMode(false);
				return true;
			}
		}
	}
	if (isReadOnly() && e->type() == QEvent::MouseButtonDblClick) {
		setEditableMode(true);
		//grabWidget = (QWidget*)obj;
		//grabWidget->grabMouse();
		return true;
	}

	return KTextBrowser::eventFilter(obj, e);
}

void TopicView::bgColorChanged()
{
	setPaletteBackgroundColor (KwircSettings::bgColor());
}

void TopicView::textColorChanged()
{
	setPaletteForegroundColor (KwircSettings::textColor());
}
